#!/usr/bin/env python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This demonstrates how to add an ad group bid modifier for mobile devices.

To get ad group bid modifiers, run get_ad_group_bid_modifiers.py
"""


import argparse
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.enums.types.device import DeviceEnum
from google.ads.googleads.v22.resources.types.ad_group_bid_modifier import (
    AdGroupBidModifier,
)
from google.ads.googleads.v22.services.services.ad_group_bid_modifier_service import (
    AdGroupBidModifierServiceClient,
)
from google.ads.googleads.v22.services.services.ad_group_service import (
    AdGroupServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_bid_modifier_service import (
    AdGroupBidModifierOperation,
    MutateAdGroupBidModifiersResponse,
)


# [START add_ad_group_bid_modifier]
def main(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    bid_modifier_value: float,
) -> None:
    ad_group_service: AdGroupServiceClient = client.get_service(
        "AdGroupService"
    )
    ad_group_bm_service: AdGroupBidModifierServiceClient = client.get_service(
        "AdGroupBidModifierService"
    )

    # Create ad group bid modifier for mobile devices with the specified ad
    # group ID and bid modifier value.
    ad_group_bid_modifier_operation: AdGroupBidModifierOperation = (
        client.get_type("AdGroupBidModifierOperation")
    )
    ad_group_bid_modifier: AdGroupBidModifier = (
        ad_group_bid_modifier_operation.create
    )

    # Set the ad group.
    ad_group_bid_modifier.ad_group = ad_group_service.ad_group_path(
        customer_id, ad_group_id
    )

    # Set the bid modifier.
    ad_group_bid_modifier.bid_modifier = bid_modifier_value

    # Sets the device.
    device_enum: DeviceEnum = client.enums.DeviceEnum
    ad_group_bid_modifier.device.type_ = device_enum.MOBILE

    # Add the ad group bid modifier.
    ad_group_bm_response: MutateAdGroupBidModifiersResponse = (
        ad_group_bm_service.mutate_ad_group_bid_modifiers(
            customer_id=customer_id,
            operations=[ad_group_bid_modifier_operation],
        )
    )
    # [END add_ad_group_bid_modifier]

    print(
        "Created ad group bid modifier: "
        f"{ad_group_bm_response.results[0].resource_name}."
    )


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=(
            "Adds an ad group bid modifier to the specified ad group "
            "ID, for the given customer ID."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a", "--ad_group_id", type=str, required=True, help="The ad group ID."
    )
    parser.add_argument(
        "-b",
        "--bid_modifier_value",
        type=float,
        required=False,
        default=1.5,
        help="The bid modifier value.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.ad_group_id,
            args.bid_modifier_value,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.enums.types import search_term_targeting_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "SearchTermView",
    },
)


class SearchTermView(proto.Message):
    r"""A search term view with metrics aggregated by search term at
    the ad group level. This view does not include Performance Max
    data. If you are looking for Performance Max search term data,
    use the CampaignSearchTermView instead.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the search term view.
            Search term view resource names have the form:

            ``customers/{customer_id}/searchTermViews/{campaign_id}~{ad_group_id}~{URL-base64_search_term}``
        search_term (str):
            Output only. The search term.

            This field is a member of `oneof`_ ``_search_term``.
        ad_group (str):
            Output only. The ad group the search term
            served in.

            This field is a member of `oneof`_ ``_ad_group``.
        status (google.ads.googleads.v21.enums.types.SearchTermTargetingStatusEnum.SearchTermTargetingStatus):
            Output only. Indicates whether the search
            term is currently one of your targeted or
            excluded keywords.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    search_term: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    ad_group: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    status: (
        search_term_targeting_status.SearchTermTargetingStatusEnum.SearchTermTargetingStatus
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=search_term_targeting_status.SearchTermTargetingStatusEnum.SearchTermTargetingStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "ResourceCountLimitExceededErrorEnum",
    },
)


class ResourceCountLimitExceededErrorEnum(proto.Message):
    r"""Container for enum describing possible resource count limit
    exceeded errors.

    """

    class ResourceCountLimitExceededError(proto.Enum):
        r"""Enum describing possible resource count limit exceeded
        errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            ACCOUNT_LIMIT (2):
                Indicates that this request would exceed the
                number of allowed resources for the Google Ads
                account. The exact resource type and limit being
                checked can be inferred from accountLimitType.
            CAMPAIGN_LIMIT (3):
                Indicates that this request would exceed the
                number of allowed resources in a Campaign. The
                exact resource type and limit being checked can
                be inferred from accountLimitType, and the
                numeric id of the Campaign involved is given by
                enclosingId.
            ADGROUP_LIMIT (4):
                Indicates that this request would exceed the
                number of allowed resources in an ad group. The
                exact resource type and limit being checked can
                be inferred from accountLimitType, and the
                numeric id of the ad group involved is given by
                enclosingId.
            AD_GROUP_AD_LIMIT (5):
                Indicates that this request would exceed the
                number of allowed resources in an ad group ad.
                The exact resource type and limit being checked
                can be inferred from accountLimitType, and the
                enclosingId contains the ad group id followed by
                the ad id, separated by a single comma (,).
            AD_GROUP_CRITERION_LIMIT (6):
                Indicates that this request would exceed the
                number of allowed resources in an ad group
                criterion. The exact resource type and limit
                being checked can be inferred from
                accountLimitType, and the enclosingId contains
                the ad group id followed by the criterion id,
                separated by a single comma (,).
            SHARED_SET_LIMIT (7):
                Indicates that this request would exceed the
                number of allowed resources in this shared set.
                The exact resource type and limit being checked
                can be inferred from accountLimitType, and the
                numeric id of the shared set involved is given
                by enclosingId.
            MATCHING_FUNCTION_LIMIT (8):
                Exceeds a limit related to a matching
                function.
            RESPONSE_ROW_LIMIT_EXCEEDED (9):
                The response for this request would exceed
                the maximum number of rows that can be returned.
            RESOURCE_LIMIT (10):
                This request would exceed a limit on the
                number of allowed resources. The details of
                which type of limit was exceeded will eventually
                be returned in ErrorDetails.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ACCOUNT_LIMIT = 2
        CAMPAIGN_LIMIT = 3
        ADGROUP_LIMIT = 4
        AD_GROUP_AD_LIMIT = 5
        AD_GROUP_CRITERION_LIMIT = 6
        SHARED_SET_LIMIT = 7
        MATCHING_FUNCTION_LIMIT = 8
        RESPONSE_ROW_LIMIT_EXCEEDED = 9
        RESOURCE_LIMIT = 10


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "LocalServicesBusinessRegistrationCheckRejectionReasonEnum",
    },
)


class LocalServicesBusinessRegistrationCheckRejectionReasonEnum(proto.Message):
    r"""Container for enum describing the rejection reason of a local
    services business registration check verification artifact.

    """

    class LocalServicesBusinessRegistrationCheckRejectionReason(proto.Enum):
        r"""Enums describing possible rejection reasons of a local
        services business registration check verification artifact.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            BUSINESS_NAME_MISMATCH (2):
                Business name doesn't match business name for
                the Local Services Ad.
            BUSINESS_DETAILS_MISMATCH (3):
                Business details mismatch.
            ID_NOT_FOUND (4):
                Business registration ID not found.
            POOR_DOCUMENT_IMAGE_QUALITY (5):
                Uploaded document not clear, blurry, etc.
            DOCUMENT_EXPIRED (6):
                Uploaded document has expired.
            DOCUMENT_INVALID (7):
                Document revoked or annuled.
            DOCUMENT_TYPE_MISMATCH (8):
                Document type mismatch.
            DOCUMENT_UNVERIFIABLE (9):
                Uploaded document could not be verified as
                legitimate.
            OTHER (10):
                The business registration process could not
                be completed due to an issue. Contact
                https://support.google.com/localservices to
                learn more.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        BUSINESS_NAME_MISMATCH = 2
        BUSINESS_DETAILS_MISMATCH = 3
        ID_NOT_FOUND = 4
        POOR_DOCUMENT_IMAGE_QUALITY = 5
        DOCUMENT_EXPIRED = 6
        DOCUMENT_INVALID = 7
        DOCUMENT_TYPE_MISMATCH = 8
        DOCUMENT_UNVERIFIABLE = 9
        OTHER = 10


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import (
    placement_type as gage_placement_type,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "DetailPlacementView",
    },
)


class DetailPlacementView(proto.Message):
    r"""A view with metrics aggregated by ad group and URL or YouTube
    video.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the detail placement view.
            Detail placement view resource names have the form:

            ``customers/{customer_id}/detailPlacementViews/{ad_group_id}~{base64_placement}``
        placement (str):
            Output only. The automatic placement string
            at detail level, e. g. website URL, mobile
            application ID, or a YouTube video ID.

            This field is a member of `oneof`_ ``_placement``.
        display_name (str):
            Output only. The display name is URL name for
            websites, YouTube video name for YouTube videos,
            and translated mobile app name for mobile apps.

            This field is a member of `oneof`_ ``_display_name``.
        group_placement_target_url (str):
            Output only. URL of the group placement, for
            example, domain, link to the mobile application
            in app store, or a YouTube channel URL.

            This field is a member of `oneof`_ ``_group_placement_target_url``.
        target_url (str):
            Output only. URL of the placement, for
            example, website, link to the mobile application
            in app store, or a YouTube video URL.

            This field is a member of `oneof`_ ``_target_url``.
        placement_type (google.ads.googleads.v20.enums.types.PlacementTypeEnum.PlacementType):
            Output only. Type of the placement, for
            example, Website, YouTube Video, and Mobile
            Application.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    placement: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    group_placement_target_url: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    target_url: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    placement_type: gage_placement_type.PlacementTypeEnum.PlacementType = (
        proto.Field(
            proto.ENUM,
            number=6,
            enum=gage_placement_type.PlacementTypeEnum.PlacementType,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "GoogleAdsFieldDataTypeEnum",
    },
)


class GoogleAdsFieldDataTypeEnum(proto.Message):
    r"""Container holding the various data types."""

    class GoogleAdsFieldDataType(proto.Enum):
        r"""These are the various types a GoogleAdsService artifact may
        take on.

        Values:
            UNSPECIFIED (0):
                Unspecified
            UNKNOWN (1):
                Unknown
            BOOLEAN (2):
                Maps to google.protobuf.BoolValue

                Applicable operators:  =, !=
            DATE (3):
                Maps to google.protobuf.StringValue. It can
                be compared using the set of operators specific
                to dates however.

                Applicable operators:  =, <, >, <=, >=, BETWEEN,
                DURING, and IN
            DOUBLE (4):
                Maps to google.protobuf.DoubleValue

                Applicable operators:  =, !=, <, >, IN, NOT IN
            ENUM (5):
                Maps to an enum. It's specific definition can be found at
                type_url.

                Applicable operators: =, !=, IN, NOT IN
            FLOAT (6):
                Maps to google.protobuf.FloatValue

                Applicable operators:  =, !=, <, >, IN, NOT IN
            INT32 (7):
                Maps to google.protobuf.Int32Value

                Applicable operators:  =, !=, <, >, <=, >=,
                BETWEEN, IN, NOT IN
            INT64 (8):
                Maps to google.protobuf.Int64Value

                Applicable operators:  =, !=, <, >, <=, >=,
                BETWEEN, IN, NOT IN
            MESSAGE (9):
                Maps to a protocol buffer message type. The data type's
                details can be found in type_url.

                No operators work with MESSAGE fields.
            RESOURCE_NAME (10):
                Maps to google.protobuf.StringValue. Represents the resource
                name (unique id) of a resource or one of its foreign keys.

                No operators work with RESOURCE_NAME fields.
            STRING (11):
                Maps to google.protobuf.StringValue.

                Applicable operators:  =, !=, LIKE, NOT LIKE,
                IN, NOT IN
            UINT64 (12):
                Maps to google.protobuf.UInt64Value

                Applicable operators:  =, !=, <, >, <=, >=,
                BETWEEN, IN, NOT IN
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        BOOLEAN = 2
        DATE = 3
        DOUBLE = 4
        ENUM = 5
        FLOAT = 6
        INT32 = 7
        INT64 = 8
        MESSAGE = 9
        RESOURCE_NAME = 10
        STRING = 11
        UINT64 = 12


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.resources.types import keyword_theme_constant


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "SuggestKeywordThemeConstantsRequest",
        "SuggestKeywordThemeConstantsResponse",
    },
)


class SuggestKeywordThemeConstantsRequest(proto.Message):
    r"""Request message for
    [KeywordThemeConstantService.SuggestKeywordThemeConstants][google.ads.googleads.v21.services.KeywordThemeConstantService.SuggestKeywordThemeConstants].

    Attributes:
        query_text (str):
            The query text of a keyword theme that will
            be used to map to similar keyword themes. For
            example, "plumber" or "roofer".
        country_code (str):
            Upper-case, two-letter country code as
            defined by ISO-3166. This for refining the scope
            of the query, default to 'US' if not set.
        language_code (str):
            The two letter language code for get
            corresponding keyword theme for refining the
            scope of the query, default to 'en' if not set.
    """

    query_text: str = proto.Field(
        proto.STRING,
        number=1,
    )
    country_code: str = proto.Field(
        proto.STRING,
        number=2,
    )
    language_code: str = proto.Field(
        proto.STRING,
        number=3,
    )


class SuggestKeywordThemeConstantsResponse(proto.Message):
    r"""Response message for
    [KeywordThemeConstantService.SuggestKeywordThemeConstants][google.ads.googleads.v21.services.KeywordThemeConstantService.SuggestKeywordThemeConstants].

    Attributes:
        keyword_theme_constants (MutableSequence[google.ads.googleads.v21.resources.types.KeywordThemeConstant]):
            Smart Campaign keyword theme suggestions.
    """

    keyword_theme_constants: MutableSequence[
        keyword_theme_constant.KeywordThemeConstant
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=keyword_theme_constant.KeywordThemeConstant,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.common.types import custom_parameter
from google.ads.googleads.v19.common.types import (
    targeting_setting as gagc_targeting_setting,
)
from google.ads.googleads.v19.enums.types import ad_group_ad_rotation_mode
from google.ads.googleads.v19.enums.types import ad_group_primary_status
from google.ads.googleads.v19.enums.types import ad_group_primary_status_reason
from google.ads.googleads.v19.enums.types import ad_group_status
from google.ads.googleads.v19.enums.types import ad_group_type
from google.ads.googleads.v19.enums.types import asset_field_type
from google.ads.googleads.v19.enums.types import asset_set_type
from google.ads.googleads.v19.enums.types import bidding_source
from google.ads.googleads.v19.enums.types import demand_gen_channel_config
from google.ads.googleads.v19.enums.types import demand_gen_channel_strategy
from google.ads.googleads.v19.enums.types import targeting_dimension


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "AdGroup",
    },
)


class AdGroup(proto.Message):
    r"""An ad group.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the ad group. Ad group
            resource names have the form:

            ``customers/{customer_id}/adGroups/{ad_group_id}``
        id (int):
            Output only. The ID of the ad group.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            The name of the ad group.

            This field is required and should not be empty
            when creating new ad groups.

            It must contain fewer than 255 UTF-8 full-width
            characters.

            It must not contain any null (code point 0x0),
            NL line feed (code point 0xA) or carriage return
            (code point 0xD) characters.

            This field is a member of `oneof`_ ``_name``.
        status (google.ads.googleads.v19.enums.types.AdGroupStatusEnum.AdGroupStatus):
            The status of the ad group.
        type_ (google.ads.googleads.v19.enums.types.AdGroupTypeEnum.AdGroupType):
            Immutable. The type of the ad group.
        ad_rotation_mode (google.ads.googleads.v19.enums.types.AdGroupAdRotationModeEnum.AdGroupAdRotationMode):
            The ad rotation mode of the ad group.
        base_ad_group (str):
            Output only. For draft or experiment ad
            groups, this field is the resource name of the
            base ad group from which this ad group was
            created. If a draft or experiment ad group does
            not have a base ad group, then this field is
            null.

            For base ad groups, this field equals the ad
            group resource name.

            This field is read-only.

            This field is a member of `oneof`_ ``_base_ad_group``.
        tracking_url_template (str):
            The URL template for constructing a tracking
            URL.

            This field is a member of `oneof`_ ``_tracking_url_template``.
        url_custom_parameters (MutableSequence[google.ads.googleads.v19.common.types.CustomParameter]):
            The list of mappings used to substitute custom parameter
            tags in a ``tracking_url_template``, ``final_urls``, or
            ``mobile_final_urls``.
        campaign (str):
            Immutable. The campaign to which the ad group
            belongs.

            This field is a member of `oneof`_ ``_campaign``.
        cpc_bid_micros (int):
            The maximum CPC (cost-per-click) bid.

            This field is a member of `oneof`_ ``_cpc_bid_micros``.
        effective_cpc_bid_micros (int):
            Output only. Value will be same as that of
            the CPC (cost-per-click) bid value when the
            bidding strategy is one of manual cpc, enhanced
            cpc, page one promoted or target outrank share,
            otherwise the value will be null.

            This field is a member of `oneof`_ ``_effective_cpc_bid_micros``.
        cpm_bid_micros (int):
            The maximum CPM (cost-per-thousand viewable
            impressions) bid.

            This field is a member of `oneof`_ ``_cpm_bid_micros``.
        target_cpa_micros (int):
            The target CPA (cost-per-acquisition). If the ad group's
            campaign bidding strategy is TargetCpa or
            MaximizeConversions (with its target_cpa field set), then
            this field overrides the target CPA specified in the
            campaign's bidding strategy. Otherwise, this value is
            ignored.

            This field is a member of `oneof`_ ``_target_cpa_micros``.
        cpv_bid_micros (int):
            The CPV (cost-per-view) bid.

            This field is a member of `oneof`_ ``_cpv_bid_micros``.
        target_cpm_micros (int):
            Average amount in micros that the advertiser
            is willing to pay for every thousand times the
            ad is shown.

            This field is a member of `oneof`_ ``_target_cpm_micros``.
        target_roas (float):
            The target ROAS (return-on-ad-spend) override. If the ad
            group's campaign bidding strategy is TargetRoas or
            MaximizeConversionValue (with its target_roas field set),
            then this field overrides the target ROAS specified in the
            campaign's bidding strategy. Otherwise, this value is
            ignored.

            This field is a member of `oneof`_ ``_target_roas``.
        percent_cpc_bid_micros (int):
            The percent cpc bid amount, expressed as a fraction of the
            advertised price for some good or service. The valid range
            for the fraction is [0,1) and the value stored here is
            1,000,000 \* [fraction].

            This field is a member of `oneof`_ ``_percent_cpc_bid_micros``.
        fixed_cpm_micros (int):
            The fixed amount in micros that the
            advertiser pays for every thousand impressions
            of the ad.

            This field is a member of `oneof`_ ``_fixed_cpm_micros``.
        target_cpv_micros (int):
            Average amount in micros that the advertiser
            is willing to pay for every ad view.

            This field is a member of `oneof`_ ``_target_cpv_micros``.
        optimized_targeting_enabled (bool):
            True if optimized targeting is enabled.
            Optimized Targeting is the replacement for
            Audience Expansion.
        exclude_demographic_expansion (bool):
            When this value is true, demographics will be excluded from
            the types of targeting which are expanded when
            optimized_targeting_enabled is true. When
            optimized_targeting_enabled is false, this field is ignored.
            Default is false.
        display_custom_bid_dimension (google.ads.googleads.v19.enums.types.TargetingDimensionEnum.TargetingDimension):
            Allows advertisers to specify a targeting
            dimension on which to place absolute bids. This
            is only applicable for campaigns that target
            only the display network and not search.
        final_url_suffix (str):
            URL template for appending params to Final
            URL.

            This field is a member of `oneof`_ ``_final_url_suffix``.
        targeting_setting (google.ads.googleads.v19.common.types.TargetingSetting):
            Setting for targeting related features.
        audience_setting (google.ads.googleads.v19.resources.types.AdGroup.AudienceSetting):
            Immutable. Setting for audience related
            features.
        effective_target_cpa_micros (int):
            Output only. The effective target CPA
            (cost-per-acquisition). This field is read-only.

            This field is a member of `oneof`_ ``_effective_target_cpa_micros``.
        effective_target_cpa_source (google.ads.googleads.v19.enums.types.BiddingSourceEnum.BiddingSource):
            Output only. Source of the effective target
            CPA. This field is read-only.
        effective_target_roas (float):
            Output only. The effective target ROAS
            (return-on-ad-spend). This field is read-only.

            This field is a member of `oneof`_ ``_effective_target_roas``.
        effective_target_roas_source (google.ads.googleads.v19.enums.types.BiddingSourceEnum.BiddingSource):
            Output only. Source of the effective target
            ROAS. This field is read-only.
        labels (MutableSequence[str]):
            Output only. The resource names of labels
            attached to this ad group.
        excluded_parent_asset_field_types (MutableSequence[google.ads.googleads.v19.enums.types.AssetFieldTypeEnum.AssetFieldType]):
            The asset field types that should be excluded
            from this ad group. Asset links with these field
            types will not be inherited by this ad group
            from the upper levels.
        excluded_parent_asset_set_types (MutableSequence[google.ads.googleads.v19.enums.types.AssetSetTypeEnum.AssetSetType]):
            The asset set types that should be excluded from this ad
            group. Asset set links with these types will not be
            inherited by this ad group from the upper levels. Location
            group types (GMB_DYNAMIC_LOCATION_GROUP,
            CHAIN_DYNAMIC_LOCATION_GROUP, and STATIC_LOCATION_GROUP) are
            child types of LOCATION_SYNC. Therefore, if LOCATION_SYNC is
            set for this field, all location group asset sets are not
            allowed to be linked to this ad group, and all Location
            Extension (LE) and Affiliate Location Extensions (ALE) will
            not be served under this ad group. Only LOCATION_SYNC is
            currently supported.
        primary_status (google.ads.googleads.v19.enums.types.AdGroupPrimaryStatusEnum.AdGroupPrimaryStatus):
            Output only. Provides aggregated view into
            why an ad group is not serving or not serving
            optimally.
        primary_status_reasons (MutableSequence[google.ads.googleads.v19.enums.types.AdGroupPrimaryStatusReasonEnum.AdGroupPrimaryStatusReason]):
            Output only. Provides reasons for why an ad
            group is not serving or not serving optimally.
        demand_gen_ad_group_settings (google.ads.googleads.v19.resources.types.AdGroup.DemandGenAdGroupSettings):
            Settings for Demand Gen ad groups.
    """

    class AudienceSetting(proto.Message):
        r"""Settings for the audience targeting.

        Attributes:
            use_audience_grouped (bool):
                Immutable. If true, this ad group uses an
                Audience resource for audience targeting. If
                false, this ad group may use audience segment
                criteria instead.
        """

        use_audience_grouped: bool = proto.Field(
            proto.BOOL,
            number=1,
        )

    class DemandGenAdGroupSettings(proto.Message):
        r"""Settings for Demand Gen ad groups.

        Attributes:
            channel_controls (google.ads.googleads.v19.resources.types.AdGroup.DemandGenAdGroupSettings.DemandGenChannelControls):
                Channel controls for Demand Gen ad groups.
        """

        class DemandGenChannelControls(proto.Message):
            r"""Channel controls for Demand Gen ad groups.

            This message has `oneof`_ fields (mutually exclusive fields).
            For each oneof, at most one member field can be set at the same time.
            Setting any member of the oneof automatically clears all other
            members.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                channel_config (google.ads.googleads.v19.enums.types.DemandGenChannelConfigEnum.DemandGenChannelConfig):
                    Output only. Channel configuration reflecting
                    which field in the oneof is populated.
                channel_strategy (google.ads.googleads.v19.enums.types.DemandGenChannelStrategyEnum.DemandGenChannelStrategy):
                    High level channel strategy.

                    This field is a member of `oneof`_ ``channel_configuration``.
                selected_channels (google.ads.googleads.v19.resources.types.AdGroup.DemandGenAdGroupSettings.DemandGenChannelControls.DemandGenSelectedChannels):
                    Explicitly selected channels. This field
                    should be set with at least one true value when
                    present.

                    This field is a member of `oneof`_ ``channel_configuration``.
            """

            class DemandGenSelectedChannels(proto.Message):
                r"""Explicitly selected channels for channel controls in Demand
                Gen ad groups.

                Attributes:
                    youtube_in_stream (bool):
                        Whether to enable ads on the YouTube
                        In-Stream channel.
                    youtube_in_feed (bool):
                        Whether to enable ads on the YouTube In-Feed
                        channel.
                    youtube_shorts (bool):
                        Whether to enable ads on the YouTube Shorts
                        channel.
                    discover (bool):
                        Whether to enable ads on the Discover
                        channel.
                    gmail (bool):
                        Whether to enable ads on the Gmail channel.
                    display (bool):
                        Whether to enable ads on the Display channel.
                """

                youtube_in_stream: bool = proto.Field(
                    proto.BOOL,
                    number=1,
                )
                youtube_in_feed: bool = proto.Field(
                    proto.BOOL,
                    number=2,
                )
                youtube_shorts: bool = proto.Field(
                    proto.BOOL,
                    number=3,
                )
                discover: bool = proto.Field(
                    proto.BOOL,
                    number=4,
                )
                gmail: bool = proto.Field(
                    proto.BOOL,
                    number=5,
                )
                display: bool = proto.Field(
                    proto.BOOL,
                    number=6,
                )

            channel_config: (
                demand_gen_channel_config.DemandGenChannelConfigEnum.DemandGenChannelConfig
            ) = proto.Field(
                proto.ENUM,
                number=1,
                enum=demand_gen_channel_config.DemandGenChannelConfigEnum.DemandGenChannelConfig,
            )
            channel_strategy: (
                demand_gen_channel_strategy.DemandGenChannelStrategyEnum.DemandGenChannelStrategy
            ) = proto.Field(
                proto.ENUM,
                number=2,
                oneof="channel_configuration",
                enum=demand_gen_channel_strategy.DemandGenChannelStrategyEnum.DemandGenChannelStrategy,
            )
            selected_channels: "AdGroup.DemandGenAdGroupSettings.DemandGenChannelControls.DemandGenSelectedChannels" = proto.Field(
                proto.MESSAGE,
                number=3,
                oneof="channel_configuration",
                message="AdGroup.DemandGenAdGroupSettings.DemandGenChannelControls.DemandGenSelectedChannels",
            )

        channel_controls: (
            "AdGroup.DemandGenAdGroupSettings.DemandGenChannelControls"
        ) = proto.Field(
            proto.MESSAGE,
            number=1,
            message="AdGroup.DemandGenAdGroupSettings.DemandGenChannelControls",
        )

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=34,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=35,
        optional=True,
    )
    status: ad_group_status.AdGroupStatusEnum.AdGroupStatus = proto.Field(
        proto.ENUM,
        number=5,
        enum=ad_group_status.AdGroupStatusEnum.AdGroupStatus,
    )
    type_: ad_group_type.AdGroupTypeEnum.AdGroupType = proto.Field(
        proto.ENUM,
        number=12,
        enum=ad_group_type.AdGroupTypeEnum.AdGroupType,
    )
    ad_rotation_mode: (
        ad_group_ad_rotation_mode.AdGroupAdRotationModeEnum.AdGroupAdRotationMode
    ) = proto.Field(
        proto.ENUM,
        number=22,
        enum=ad_group_ad_rotation_mode.AdGroupAdRotationModeEnum.AdGroupAdRotationMode,
    )
    base_ad_group: str = proto.Field(
        proto.STRING,
        number=36,
        optional=True,
    )
    tracking_url_template: str = proto.Field(
        proto.STRING,
        number=37,
        optional=True,
    )
    url_custom_parameters: MutableSequence[custom_parameter.CustomParameter] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=6,
            message=custom_parameter.CustomParameter,
        )
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=38,
        optional=True,
    )
    cpc_bid_micros: int = proto.Field(
        proto.INT64,
        number=39,
        optional=True,
    )
    effective_cpc_bid_micros: int = proto.Field(
        proto.INT64,
        number=57,
        optional=True,
    )
    cpm_bid_micros: int = proto.Field(
        proto.INT64,
        number=40,
        optional=True,
    )
    target_cpa_micros: int = proto.Field(
        proto.INT64,
        number=41,
        optional=True,
    )
    cpv_bid_micros: int = proto.Field(
        proto.INT64,
        number=42,
        optional=True,
    )
    target_cpm_micros: int = proto.Field(
        proto.INT64,
        number=43,
        optional=True,
    )
    target_roas: float = proto.Field(
        proto.DOUBLE,
        number=44,
        optional=True,
    )
    percent_cpc_bid_micros: int = proto.Field(
        proto.INT64,
        number=45,
        optional=True,
    )
    fixed_cpm_micros: int = proto.Field(
        proto.INT64,
        number=64,
        optional=True,
    )
    target_cpv_micros: int = proto.Field(
        proto.INT64,
        number=65,
        optional=True,
    )
    optimized_targeting_enabled: bool = proto.Field(
        proto.BOOL,
        number=59,
    )
    exclude_demographic_expansion: bool = proto.Field(
        proto.BOOL,
        number=67,
    )
    display_custom_bid_dimension: (
        targeting_dimension.TargetingDimensionEnum.TargetingDimension
    ) = proto.Field(
        proto.ENUM,
        number=23,
        enum=targeting_dimension.TargetingDimensionEnum.TargetingDimension,
    )
    final_url_suffix: str = proto.Field(
        proto.STRING,
        number=46,
        optional=True,
    )
    targeting_setting: gagc_targeting_setting.TargetingSetting = proto.Field(
        proto.MESSAGE,
        number=25,
        message=gagc_targeting_setting.TargetingSetting,
    )
    audience_setting: AudienceSetting = proto.Field(
        proto.MESSAGE,
        number=56,
        message=AudienceSetting,
    )
    effective_target_cpa_micros: int = proto.Field(
        proto.INT64,
        number=47,
        optional=True,
    )
    effective_target_cpa_source: (
        bidding_source.BiddingSourceEnum.BiddingSource
    ) = proto.Field(
        proto.ENUM,
        number=29,
        enum=bidding_source.BiddingSourceEnum.BiddingSource,
    )
    effective_target_roas: float = proto.Field(
        proto.DOUBLE,
        number=48,
        optional=True,
    )
    effective_target_roas_source: (
        bidding_source.BiddingSourceEnum.BiddingSource
    ) = proto.Field(
        proto.ENUM,
        number=32,
        enum=bidding_source.BiddingSourceEnum.BiddingSource,
    )
    labels: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=49,
    )
    excluded_parent_asset_field_types: MutableSequence[
        asset_field_type.AssetFieldTypeEnum.AssetFieldType
    ] = proto.RepeatedField(
        proto.ENUM,
        number=54,
        enum=asset_field_type.AssetFieldTypeEnum.AssetFieldType,
    )
    excluded_parent_asset_set_types: MutableSequence[
        asset_set_type.AssetSetTypeEnum.AssetSetType
    ] = proto.RepeatedField(
        proto.ENUM,
        number=58,
        enum=asset_set_type.AssetSetTypeEnum.AssetSetType,
    )
    primary_status: (
        ad_group_primary_status.AdGroupPrimaryStatusEnum.AdGroupPrimaryStatus
    ) = proto.Field(
        proto.ENUM,
        number=62,
        enum=ad_group_primary_status.AdGroupPrimaryStatusEnum.AdGroupPrimaryStatus,
    )
    primary_status_reasons: MutableSequence[
        ad_group_primary_status_reason.AdGroupPrimaryStatusReasonEnum.AdGroupPrimaryStatusReason
    ] = proto.RepeatedField(
        proto.ENUM,
        number=63,
        enum=ad_group_primary_status_reason.AdGroupPrimaryStatusReasonEnum.AdGroupPrimaryStatusReason,
    )
    demand_gen_ad_group_settings: DemandGenAdGroupSettings = proto.Field(
        proto.MESSAGE,
        number=91,
        message=DemandGenAdGroupSettings,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.common.types import audiences
from google.ads.googleads.v19.enums.types import audience_scope
from google.ads.googleads.v19.enums.types import audience_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "Audience",
    },
)


class Audience(proto.Message):
    r"""Audience is an effective targeting option that lets you
    intersect different segment attributes, such as detailed
    demographics and affinities, to create audiences that represent
    sections of your target segments.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the audience. Audience names
            have the form:

            ``customers/{customer_id}/audiences/{audience_id}``
        id (int):
            Output only. ID of the audience.
        status (google.ads.googleads.v19.enums.types.AudienceStatusEnum.AudienceStatus):
            Output only. Status of this audience.
            Indicates whether the audience is enabled or
            removed.
        name (str):
            Name of the audience. It should be unique across all
            audiences within the account. It must have a minimum length
            of 1 and maximum length of 255. Required when scope is not
            set or is set to CUSTOMER. Cannot be set or updated when
            scope is ASSET_GROUP.

            This field is a member of `oneof`_ ``_name``.
        description (str):
            Description of this audience.
        dimensions (MutableSequence[google.ads.googleads.v19.common.types.AudienceDimension]):
            Positive dimensions specifying the audience
            composition.
        exclusion_dimension (google.ads.googleads.v19.common.types.AudienceExclusionDimension):
            Negative dimension specifying the audience
            composition.
        scope (google.ads.googleads.v19.enums.types.AudienceScopeEnum.AudienceScope):
            Defines the scope this audience can be used in. By default,
            the scope is CUSTOMER. Audiences can be created with a scope
            of ASSET_GROUP for exclusive use by a single asset_group.
            Scope may change from ASSET_GROUP to CUSTOMER but not from
            CUSTOMER to ASSET_GROUP.
        asset_group (str):
            Immutable. The asset group that this audience is scoped
            under. Must be set if and only if scope is ASSET_GROUP.
            Immutable after creation. If an audience with ASSET_GROUP
            scope is upgraded to CUSTOMER scope, this field will
            automatically be cleared.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    status: audience_status.AudienceStatusEnum.AudienceStatus = proto.Field(
        proto.ENUM,
        number=3,
        enum=audience_status.AudienceStatusEnum.AudienceStatus,
    )
    name: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    description: str = proto.Field(
        proto.STRING,
        number=5,
    )
    dimensions: MutableSequence[audiences.AudienceDimension] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=6,
            message=audiences.AudienceDimension,
        )
    )
    exclusion_dimension: audiences.AudienceExclusionDimension = proto.Field(
        proto.MESSAGE,
        number=7,
        message=audiences.AudienceExclusionDimension,
    )
    scope: audience_scope.AudienceScopeEnum.AudienceScope = proto.Field(
        proto.ENUM,
        number=8,
        enum=audience_scope.AudienceScopeEnum.AudienceScope,
    )
    asset_group: str = proto.Field(
        proto.STRING,
        number=9,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "DataDrivenModelStatusEnum",
    },
)


class DataDrivenModelStatusEnum(proto.Message):
    r"""Container for enum indicating data driven model status."""

    class DataDrivenModelStatus(proto.Enum):
        r"""Enumerates data driven model statuses.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            AVAILABLE (2):
                The data driven model is available.
            STALE (3):
                The data driven model is stale. It hasn't
                been updated for at least 7 days. It is still
                being used, but will become expired if it does
                not get updated for 30 days.
            EXPIRED (4):
                The data driven model expired. It hasn't been
                updated for at least 30 days and cannot be used.
                Most commonly this is because there hasn't been
                the required number of events in a recent 30-day
                period.
            NEVER_GENERATED (5):
                The data driven model has never been
                generated. Most commonly this is because there
                has never been the required number of events in
                any 30-day period.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        AVAILABLE = 2
        STALE = 3
        EXPIRED = 4
        NEVER_GENERATED = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "SlotEnum",
    },
)


class SlotEnum(proto.Message):
    r"""Container for enumeration of possible positions of the Ad."""

    class Slot(proto.Enum):
        r"""Enumerates possible positions of the Ad.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            SEARCH_SIDE (2):
                Google search: Side.
            SEARCH_TOP (3):
                Google search: Top.
            SEARCH_OTHER (4):
                Google search: Other.
            CONTENT (5):
                Google Display Network.
            SEARCH_PARTNER_TOP (6):
                Search partners: Top.
            SEARCH_PARTNER_OTHER (7):
                Search partners: Other.
            MIXED (8):
                Cross-network.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        SEARCH_SIDE = 2
        SEARCH_TOP = 3
        SEARCH_OTHER = 4
        CONTENT = 5
        SEARCH_PARTNER_TOP = 6
        SEARCH_PARTNER_OTHER = 7
        MIXED = 8


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "LabelErrorEnum",
    },
)


class LabelErrorEnum(proto.Message):
    r"""Container for enum describing possible label errors."""

    class LabelError(proto.Enum):
        r"""Enum describing possible label errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CANNOT_APPLY_INACTIVE_LABEL (2):
                An inactive label cannot be applied.
            CANNOT_APPLY_LABEL_TO_DISABLED_AD_GROUP_CRITERION (3):
                A label cannot be applied to a disabled ad
                group criterion.
            CANNOT_APPLY_LABEL_TO_NEGATIVE_AD_GROUP_CRITERION (4):
                A label cannot be applied to a negative ad
                group criterion.
            EXCEEDED_LABEL_LIMIT_PER_TYPE (5):
                Cannot apply more than 50 labels per
                resource.
            INVALID_RESOURCE_FOR_MANAGER_LABEL (6):
                Labels from a manager account cannot be
                applied to campaign, ad group, ad group ad, or
                ad group criterion resources.
            DUPLICATE_NAME (7):
                Label names must be unique.
            INVALID_LABEL_NAME (8):
                Label names cannot be empty.
            CANNOT_ATTACH_LABEL_TO_DRAFT (9):
                Labels cannot be applied to a draft.
            CANNOT_ATTACH_NON_MANAGER_LABEL_TO_CUSTOMER (10):
                Labels not from a manager account cannot be
                applied to the customer resource.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CANNOT_APPLY_INACTIVE_LABEL = 2
        CANNOT_APPLY_LABEL_TO_DISABLED_AD_GROUP_CRITERION = 3
        CANNOT_APPLY_LABEL_TO_NEGATIVE_AD_GROUP_CRITERION = 4
        EXCEEDED_LABEL_LIMIT_PER_TYPE = 5
        INVALID_RESOURCE_FOR_MANAGER_LABEL = 6
        DUPLICATE_NAME = 7
        INVALID_LABEL_NAME = 8
        CANNOT_ATTACH_LABEL_TO_DRAFT = 9
        CANNOT_ATTACH_NON_MANAGER_LABEL_TO_CUSTOMER = 10


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example is to get impact metrics for a custom budget.

Use this example to get impact metrics for a given budget amount.

This example uses the following:
1) Performance Max for the campaign type
2) United States for the geo targeting
3) Maximize Conversions Value for the bidding strategy

To get budget recommendations, run generate_budget_recommendations.py.
"""


import argparse
import sys
from typing import List, Dict, Any

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.recommendation_service import (
    RecommendationServiceClient,
)
from google.ads.googleads.v22.services.types.recommendation_service import (
    GenerateRecommendationsRequest,
    GenerateRecommendationsResponse,
)
from google.ads.googleads.v22.resources.types.recommendation import (
    Recommendation,
)


def main(
    client: GoogleAdsClient, customer_id: str, user_provided_budget_amount: int
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        user_provided_budget_amount: a user-provided budget amount (not in micros), to retrieve impact metrics for.
    """
    recommendation_service: RecommendationServiceClient = client.get_service(
        "RecommendationService"
    )
    request: GenerateRecommendationsRequest = client.get_type(
        "GenerateRecommendationsRequest"
    )

    request.customer_id = customer_id
    request.recommendation_types = ["CAMPAIGN_BUDGET"]
    request.advertising_channel_type = (
        client.enums.AdvertisingChannelTypeEnum.PERFORMANCE_MAX
    )
    request.bidding_info.bidding_strategy_type = "MAXIMIZE_CONVERSION_VALUE"
    request.positive_locations_ids = [2840]  # 2840 is for United States
    request.asset_group_info = [{"final_url": "https://www.your-company.com/"}]
    # Multiply the user-provided budget by 1,000,000 to convert to micros, as required for current_budget
    request.budget_info.current_budget = round(
        (user_provided_budget_amount * 1000000), 2
    )

    results: GenerateRecommendationsResponse = (
        recommendation_service.generate_recommendations(request)
    )

    recommendations: List[Recommendation] = results.recommendations

    # List to store impact metrics for user input budget
    budget_impact_metrics: List[Dict[str, Any]] = []

    # Get impact metrics for custom budget.
    for rec in recommendations:
        campaign_budget_rec = rec.campaign_budget_recommendation
        # Loop through the budget options in the campaign budget recommendation
        # to compile a list of budget amounts and their respective potential
        # impact metrics. If you have a campaign creation interface,
        # you could display this information for end users to decide which
        # budget amount best aligns with their goals.
        for budget_option in campaign_budget_rec.budget_options:
            if hasattr(budget_option, "impact"):
                impact = budget_option.impact
                budget_amount_micros = budget_option.budget_amount_micros
                if (
                    budget_amount_micros / 1000000
                    == user_provided_budget_amount
                ):
                    budget_data: Dict[str, Any] = {
                        "budget_amount": round(
                            (budget_amount_micros / 1000000), 2
                        ),
                        "potential_metrics": impact.potential_metrics,
                    }
                    budget_impact_metrics.append(budget_data)
            else:
                print("impact metrics not found for this budget amount.")

    print(f"budget_impact_metrics:\n{budget_impact_metrics}")
    """
    budget_impact_metrics:
    [{'budget_amount': 100.0, 'potential_metrics': cost_micros: 700000000
    conversions: 12
    conversions_value: 481.12592352792007
    }]
    """


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description=("Get impact metrics for Performance Max budget.")
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-b",
        "--user_provided_budget_amount",
        type=int,
        required=True,
        help=("A budget amount (not in micros) advertiser wants to use."),
    )

    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.user_provided_budget_amount,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'Error with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.enums.types import asset_field_type
from google.ads.googleads.v22.enums.types import (
    asset_source as gage_asset_source,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "CampaignAggregateAssetView",
    },
)


class CampaignAggregateAssetView(proto.Message):
    r"""A campaign-level aggregate asset view that shows where the
    asset is linked, performamce of the asset and stats.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the campaign aggregate
            asset view. Campaign aggregate asset view resource names
            have the form:

            ``customers/{customer_id}/campaignAggregateAssetViews/{Campaign.campaign_id}~{Asset.asset_id}~{AssetLinkSource.asset_link_source}~{AssetFieldType.field_type}``
        campaign (str):
            Output only. Campaign in which the asset
            served.

            This field is a member of `oneof`_ ``_campaign``.
        asset (str):
            Output only. The ID of the asset.

            This field is a member of `oneof`_ ``_asset``.
        asset_source (google.ads.googleads.v22.enums.types.AssetSourceEnum.AssetSource):
            Output only. Source of the asset link.

            This field is a member of `oneof`_ ``_asset_source``.
        field_type (google.ads.googleads.v22.enums.types.AssetFieldTypeEnum.AssetFieldType):
            Output only. FieldType of the asset.

            This field is a member of `oneof`_ ``_field_type``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )
    asset: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )
    asset_source: gage_asset_source.AssetSourceEnum.AssetSource = proto.Field(
        proto.ENUM,
        number=4,
        optional=True,
        enum=gage_asset_source.AssetSourceEnum.AssetSource,
    )
    field_type: asset_field_type.AssetFieldTypeEnum.AssetFieldType = (
        proto.Field(
            proto.ENUM,
            number=5,
            optional=True,
            enum=asset_field_type.AssetFieldTypeEnum.AssetFieldType,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example illustrates how to get all campaigns and log details to

Google Cloud Logging using a custom logging interceptor.
"""

import argparse
import sys
from typing import Any, Iterable

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    SearchGoogleAdsStreamResponse,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    GoogleAdsRow,
)

from cloud_logging_interceptor import CloudLoggingInterceptor


def main(client: GoogleAdsClient, customer_id: str) -> None:
    # Instantiate the GoogleAdsService object with a custom interceptor.
    ga_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService",
        interceptors=[CloudLoggingInterceptor(api_version="v22")],
    )

    query: str = """
        SELECT
          campaign.id,
          campaign.name
        FROM campaign
        ORDER BY campaign.id
        LIMIT 10"""

    # Issues a search request using streaming.
    stream: Iterable[SearchGoogleAdsStreamResponse] = ga_service.search_stream(
        customer_id=customer_id, query=query
    )

    for batch in stream:
        results: Iterable[GoogleAdsRow] = batch.results
        for row in results:
            print(
                f"Campaign with ID {row.campaign.id} and name "
                f'"{row.campaign.name}" was found.'
            )


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Lists all campaigns for specified customer."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



#!/usr/bin/env python
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A set of helper functions for use in code examples."""

from datetime import datetime, timezone
import requests


def get_printable_datetime() -> str:
    """Generates a string for the current date and time in local time zone.

    The datetime string has the following format, where the trailing "sZ"
    represents three digits of milliseconds plus the timezone offset:
    %Y-%m-%dT%H:%M%S.sZ

    Here's an example output: "2023-03-23T15:45:25.211+04:00"

    Returns:
        a datetime string
    """
    return (
        datetime.now(timezone.utc)
        .astimezone()
        .isoformat(timespec="milliseconds")
    )


def get_image_bytes_from_url(url: str) -> bytes:
    """Retrieves the raw bytes of an image from a url.

    Args:
        url: The URL of the image to retrieve.

    Returns:
        Raw bytes of an image.
    """
    try:
        return requests.get(url).content
    except requests.exceptions.RequestException as e:
        print(f"Could not download image from url: {url}")
        raise e



#!/usr/bin/env python
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example shows how to create a Smart campaign.

More details on Smart campaigns can be found here:
https://support.google.com/google-ads/answer/7652860
"""

import argparse
import sys
from typing import List, Optional
from uuid import uuid4

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.ad_asset import AdTextAsset
from google.ads.googleads.v22.common.types.ad_type_infos import (
    SmartCampaignAdInfo,
)
from google.ads.googleads.v22.common.types.criteria import (
    AdScheduleInfo,
    KeywordThemeInfo,
    LocationInfo,
)
from google.ads.googleads.v22.resources.types.ad_group import AdGroup
from google.ads.googleads.v22.resources.types.ad_group_ad import AdGroupAd
from google.ads.googleads.v22.resources.types.campaign import Campaign
from google.ads.googleads.v22.resources.types.campaign_budget import (
    CampaignBudget,
)
from google.ads.googleads.v22.resources.types.campaign_criterion import (
    CampaignCriterion,
)
from google.ads.googleads.v22.resources.types.keyword_theme_constant import (
    KeywordThemeConstant,
)
from google.ads.googleads.v22.resources.types.smart_campaign_setting import (
    SmartCampaignSetting,
)
from google.ads.googleads.v22.services.types.ad_group_ad_service import (
    AdGroupAdOperation,
)
from google.ads.googleads.v22.services.types.ad_group_service import (
    AdGroupOperation,
)
from google.ads.googleads.v22.services.types.campaign_budget_service import (
    CampaignBudgetOperation,
)
from google.ads.googleads.v22.services.types.campaign_criterion_service import (
    CampaignCriterionOperation,
)
from google.ads.googleads.v22.services.services.campaign_service import (
    CampaignServiceClient,
)
from google.ads.googleads.v22.enums.types.minute_of_hour import (
    MinuteOfHourEnum,
)
from google.ads.googleads.v22.services.types.campaign_service import (
    CampaignOperation,
)
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    MutateGoogleAdsResponse,
    MutateOperation,
    MutateOperationResponse,
)
from google.ads.googleads.v22.services.services.keyword_theme_constant_service import (
    KeywordThemeConstantServiceClient,
)
from google.ads.googleads.v22.services.types.keyword_theme_constant_service import (
    SuggestKeywordThemeConstantsRequest,
    SuggestKeywordThemeConstantsResponse,
)
from google.ads.googleads.v22.services.services.smart_campaign_suggest_service import (
    SmartCampaignSuggestServiceClient,
)
from google.ads.googleads.v22.services.types.smart_campaign_suggest_service import (
    SmartCampaignSuggestionInfo,
    SuggestKeywordThemesResponse,
    SuggestKeywordThemesRequest,
    SuggestSmartCampaignAdResponse,
    SuggestSmartCampaignAdRequest,
    SuggestSmartCampaignBudgetOptionsResponse,
    SuggestSmartCampaignBudgetOptionsRequest,
)
from google.ads.googleads.v22.services.types.smart_campaign_setting_service import (
    SmartCampaignSettingOperation,
)
from google.api_core import protobuf_helpers

# Geo target constant for New York City.
_GEO_TARGET_CONSTANT = "1023191"
# Country code is a two-letter ISO-3166 code, for a list of all codes see:
# https://developers.google.com/google-ads/api/reference/data/codes-formats#expandable-16
_COUNTRY_CODE = "US"
# For a list of all language codes, see:
# https://developers.google.com/google-ads/api/reference/data/codes-formats#expandable-7
_LANGUAGE_CODE = "en"
_LANDING_PAGE_URL = "http://www.example.com"
_PHONE_NUMBER = "800-555-0100"
_BUDGET_TEMPORARY_ID = "-1"
_SMART_CAMPAIGN_TEMPORARY_ID = "-2"
_AD_GROUP_TEMPORARY_ID = "-3"
# These define the minimum number of headlines and descriptions that are
# required to create an AdGroupAd in a Smart campaign.
_REQUIRED_NUM_HEADLINES = 3
_REQUIRED_NUM_DESCRIPTIONS = 2


def main(
    client: GoogleAdsClient,
    customer_id: str,
    keyword_text: Optional[str],
    free_form_keyword_text: Optional[str],
    business_profile_location: Optional[str],
    business_name: Optional[str],
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        keyword_text: a keyword used for generating keyword themes.
        free_form_keyword_text: a keyword used to create a free-form keyword
          theme.
        business_profile_location: the ID of a Business Profile location.
        business_name: the name of a Business Profile.
    """
    # [START add_smart_campaign_12]
    # The SmartCampaignSuggestionInfo object acts as the basis for many of the
    # entities necessary to create a Smart campaign. It will be reused a number
    # of times to retrieve suggestions for keyword themes, budget amount,
    # ad creatives, and campaign criteria.
    suggestion_info: SmartCampaignSuggestionInfo = (
        get_smart_campaign_suggestion_info(
            client, business_profile_location, business_name
        )
    )

    # After creating a SmartCampaignSuggestionInfo object we first use it to
    # generate a list of keyword themes using the SuggestKeywordThemes method
    # on the SmartCampaignSuggestService. It is strongly recommended that you
    # use this strategy for generating keyword themes.
    keyword_themes: List[SuggestKeywordThemesResponse.KeywordTheme] = (
        get_keyword_theme_suggestions(client, customer_id, suggestion_info)
    )

    # If a keyword text is given, retrieve keyword theme constant suggestions
    # from the KeywordThemeConstantService, map them to KeywordThemes, and
    # append them to the existing list. This logic should ideally only be used
    # if the suggestions from the get_keyword_theme_suggestions function are
    # insufficient.
    if keyword_text:
        keyword_themes.extend(
            get_keyword_text_auto_completions(client, keyword_text)
        )

    # Map the KeywordThemes retrieved by the previous two steps to
    # KeywordThemeInfo instances.
    keyword_theme_infos: List[KeywordThemeInfo] = (
        map_keyword_themes_to_keyword_infos(client, keyword_themes)
    )

    # If a free-form keyword text is given we create a KeywordThemeInfo instance
    # from it and add it to the existing list.
    if free_form_keyword_text:
        keyword_theme_infos.append(
            get_free_form_keyword_theme_info(client, free_form_keyword_text)
        )

    # Now add the generated keyword themes to the suggestion info instance.
    suggestion_info.keyword_themes.extend(keyword_theme_infos)
    # [END add_smart_campaign_12]

    # Retrieve a budget amount suggestion.
    suggested_budget_amount: int = get_budget_suggestion(
        client, customer_id, suggestion_info
    )

    # Retrieve Smart campaign ad creative suggestions.
    ad_suggestions: SmartCampaignAdInfo = get_ad_suggestions(
        client, customer_id, suggestion_info
    )

    # [START add_smart_campaign_7]
    # The below methods create and return MutateOperations that we later
    # provide to the GoogleAdsService.Mutate method in order to create the
    # entities in a single request. Since the entities for a Smart campaign
    # are closely tied to one-another it's considered a best practice to
    # create them in a single Mutate request so they all complete successfully
    # or fail entirely, leaving no orphaned entities. See:
    # https://developers.google.com/google-ads/api/docs/mutating/overview
    campaign_budget_operation: MutateOperation = (
        create_campaign_budget_operation(
            client, customer_id, suggested_budget_amount
        )
    )
    smart_campaign_operation: MutateOperation = create_smart_campaign_operation(
        client, customer_id
    )
    smart_campaign_setting_operation: MutateOperation = (
        create_smart_campaign_setting_operation(
            client, customer_id, business_profile_location, business_name
        )
    )
    campaign_criterion_operations: List[MutateOperation] = (
        create_campaign_criterion_operations(
            client, customer_id, keyword_theme_infos, suggestion_info
        )
    )
    ad_group_operation: MutateOperation = create_ad_group_operation(
        client, customer_id
    )
    ad_group_ad_operation: MutateOperation = create_ad_group_ad_operation(
        client, customer_id, ad_suggestions
    )

    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )

    # Send the operations into a single Mutate request.
    response: MutateGoogleAdsResponse = googleads_service.mutate(
        customer_id=customer_id,
        mutate_operations=[
            # It's important to create these entities in this order because
            # they depend on each other, for example the SmartCampaignSetting
            # and ad group depend on the campaign, and the ad group ad depends
            # on the ad group.
            campaign_budget_operation,
            smart_campaign_operation,
            smart_campaign_setting_operation,
            # Expand the list of campaign criterion operations into the list of
            # other mutate operations
            *campaign_criterion_operations,
            ad_group_operation,
            ad_group_ad_operation,
        ],
    )

    print_response_details(response)
    # [END add_smart_campaign_7]


# [START add_smart_campaign_11]
def get_keyword_theme_suggestions(
    client: GoogleAdsClient,
    customer_id: str,
    suggestion_info: SmartCampaignSuggestionInfo,
) -> List[SuggestKeywordThemesResponse.KeywordTheme]:
    """Retrieves KeywordThemes using the given suggestion info.

    Here we use the SuggestKeywordThemes method, which uses all of the business
    details included in the given SmartCampaignSuggestionInfo instance to
    generate keyword theme suggestions. This is the recommended way to
    generate keyword themes because it uses detailed information about your
    business, its location, and website content to generate keyword themes.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        suggestion_info: a SmartCampaignSuggestionInfo instance with details
          about the business being advertised.

    Returns:
        a list of KeywordThemes.
    """
    smart_campaign_suggest_service: SmartCampaignSuggestServiceClient = (
        client.get_service("SmartCampaignSuggestService")
    )
    request: SuggestKeywordThemesRequest = client.get_type(
        "SuggestKeywordThemesRequest"
    )
    request.customer_id = customer_id
    request.suggestion_info = suggestion_info

    response: SuggestKeywordThemesResponse = (
        smart_campaign_suggest_service.suggest_keyword_themes(request=request)
    )

    print(
        f"Retrieved {len(response.keyword_themes)} keyword theme suggestions "
        "from the SuggestKeywordThemes method."
    )
    return response.keyword_themes
    # [END add_smart_campaign_11]


# [START add_smart_campaign]
def get_keyword_text_auto_completions(
    client: GoogleAdsClient, keyword_text: str
) -> List[SuggestKeywordThemesResponse.KeywordTheme]:
    """Retrieves KeywordThemeConstants for the given keyword text.

    These KeywordThemeConstants are derived from autocomplete data for the
    given keyword text. They are mapped to KeywordThemes before being returned.

    Args:
        client: an initialized GoogleAdsClient instance.
        keyword_text: a keyword used for generating keyword themes.

    Returns:
        a list of KeywordThemes.
    """
    keyword_theme_constant_service: KeywordThemeConstantServiceClient = (
        client.get_service("KeywordThemeConstantService")
    )
    request: SuggestKeywordThemeConstantsRequest = client.get_type(
        "SuggestKeywordThemeConstantsRequest"
    )
    request.query_text = keyword_text
    request.country_code = _COUNTRY_CODE
    request.language_code = _LANGUAGE_CODE

    response: SuggestKeywordThemeConstantsResponse = (
        keyword_theme_constant_service.suggest_keyword_theme_constants(
            request=request
        )
    )

    print(
        f"Retrieved {len(response.keyword_theme_constants)} keyword theme "
        f"constants using the keyword: '{keyword_text}'"
    )

    # Map the keyword theme constants to KeywordTheme instances for consistency
    # with the response from SmartCampaignSuggestService.SuggestKeywordThemes.
    keyword_themes: List[SuggestKeywordThemesResponse.KeywordTheme] = []
    keyword_theme_constant: KeywordThemeConstant
    for keyword_theme_constant in response.keyword_theme_constants:
        # Note that the SuggestKeywordThemesResponse.KeywordTheme is a nested
        # type and not the same as the top-level KeywordTheme message.
        keyword_theme: SuggestKeywordThemesResponse.KeywordTheme = (
            client.get_type("SuggestKeywordThemesResponse").KeywordTheme()
        )
        keyword_theme.keyword_theme_constant = keyword_theme_constant
        keyword_themes.append(keyword_theme)

    return keyword_themes
    # [END add_smart_campaign]


# [START add_smart_campaign_13]
def get_free_form_keyword_theme_info(
    client: GoogleAdsClient, free_form_keyword_text: str
) -> KeywordThemeInfo:
    """Creates a KeywordThemeInfo using the given free-form keyword text.

    Args:
        client: an initialized GoogleAdsClient instance.
        free_form_keyword_text: a keyword used to create a free-form keyword
          theme.

    Returns:
        a KeywordThemeInfo instance.
    """
    info: KeywordThemeInfo = client.get_type("KeywordThemeInfo")
    info.free_form_keyword_theme = free_form_keyword_text
    return info
    # [END add_smart_campaign_13]


def map_keyword_themes_to_keyword_infos(
    client: GoogleAdsClient,
    keyword_themes: List[SuggestKeywordThemesResponse.KeywordTheme],
) -> List[KeywordThemeInfo]:
    """Maps a list of KeywordThemes to KeywordThemeInfos.

    Args:
        client: an initialized GoogleAdsClient instance.
        keyword_themes: a list of KeywordThemes.

    Returns:
        a list of KeywordThemeInfos.
    """
    infos: List[KeywordThemeInfo] = []
    keyword_theme: SuggestKeywordThemesResponse.KeywordTheme
    for keyword_theme in keyword_themes:
        info: KeywordThemeInfo = client.get_type("KeywordThemeInfo")
        # Check if the keyword_theme_constant field is set.
        if "keyword_theme_constant" in keyword_theme:
            info.keyword_theme_constant = (
                keyword_theme.keyword_theme_constant.resource_name
            )
            infos.append(info)
        # Check if the free_form_keyword_theme field is set.
        elif "free_form_keyword_theme" in keyword_theme:
            info.free_form_keyword_theme = keyword_theme.free_form_keyword_theme
            infos.append(info)
        else:
            raise ValueError(
                f"A malformed KeywordTheme was encountered: {keyword_theme}"
            )

    return infos


# [START add_smart_campaign_9]
def get_smart_campaign_suggestion_info(
    client: GoogleAdsClient,
    business_profile_location: Optional[str],
    business_name: Optional[str],
) -> SmartCampaignSuggestionInfo:
    """Builds a SmartCampaignSuggestionInfo object with business details.

    The details are used by the SmartCampaignSuggestService to suggest a
    budget amount as well as creatives for the ad.

    Note that when retrieving ad creative suggestions it's required that the
    "final_url", "language_code" and "keyword_themes" fields are set on the
    SmartCampaignSuggestionInfo instance.

    Args:
        client: an initialized GoogleAdsClient instance.
        business_profile_location: the resource name of a Business Profile
          location.
        business_name: the name of a Business Profile.

    Returns:
        A SmartCampaignSuggestionInfo instance.
    """
    suggestion_info: SmartCampaignSuggestionInfo = client.get_type(
        "SmartCampaignSuggestionInfo"
    )

    # Add the URL of the campaign's landing page.
    suggestion_info.final_url = _LANDING_PAGE_URL

    # Add the language code for the campaign.
    suggestion_info.language_code = _LANGUAGE_CODE

    # Construct location information using the given geo target constant. It's
    # also possible to provide a geographic proximity using the "proximity"
    # field on suggestion_info, for example:
    #
    # suggestion_info.proximity.address.post_code = INSERT_POSTAL_CODE
    # suggestion_info.proximity.address.province_code = INSERT_PROVINCE_CODE
    # suggestion_info.proximity.address.country_code = INSERT_COUNTRY_CODE
    # suggestion_info.proximity.address.province_name = INSERT_PROVINCE_NAME
    # suggestion_info.proximity.address.street_address = INSERT_STREET_ADDRESS
    # suggestion_info.proximity.address.street_address2 = INSERT_STREET_ADDRESS_2
    # suggestion_info.proximity.address.city_name = INSERT_CITY_NAME
    # suggestion_info.proximity.radius = INSERT_RADIUS
    # suggestion_info.proximity.radius_units = RADIUS_UNITS
    #
    # For more information on proximities see:
    # https://developers.google.com/google-ads/api/reference/rpc/latest/ProximityInfo
    location: LocationInfo = client.get_type("LocationInfo")
    # Set the location to the resource name of the given geo target constant.
    location.geo_target_constant = client.get_service(
        "GeoTargetConstantService"
    ).geo_target_constant_path(_GEO_TARGET_CONSTANT)
    # Add the LocationInfo object to the list of locations on the
    # suggestion_info object. You have the option of providing multiple
    # locations when using location-based suggestions.
    suggestion_info.location_list.locations.append(location)

    # Set either of the business_profile_location or business_name, depending on
    # whichever is provided.
    if business_profile_location:
        suggestion_info.business_profile_location = business_profile_location
    else:
        suggestion_info.business_context.business_name = business_name

    # Add a schedule detailing which days of the week the business is open.
    # This schedule describes a schedule in which the business is open on
    # Mondays from 9am to 5pm.
    ad_schedule_info: AdScheduleInfo = client.get_type("AdScheduleInfo")
    # Set the day of this schedule as Monday.
    ad_schedule_info.day_of_week = client.enums.DayOfWeekEnum.MONDAY
    # Set the start hour to 9am.
    ad_schedule_info.start_hour = 9
    # Set the end hour to 5pm.
    ad_schedule_info.end_hour = 17
    # Set the start and end minute of zero, for example: 9:00 and 5:00.
    zero_minute_of_hour: MinuteOfHourEnum.MinuteOfHour = (
        client.enums.MinuteOfHourEnum.ZERO
    )
    ad_schedule_info.start_minute = zero_minute_of_hour
    ad_schedule_info.end_minute = zero_minute_of_hour
    suggestion_info.ad_schedules.append(ad_schedule_info)

    return suggestion_info
    # [END add_smart_campaign_9]


# [START add_smart_campaign_1]
def get_budget_suggestion(
    client: GoogleAdsClient,
    customer_id: str,
    suggestion_info: SmartCampaignSuggestionInfo,
) -> int:
    """Retrieves a suggested budget amount for a new budget.

    Using the SmartCampaignSuggestService to determine a daily budget for new
    and existing Smart campaigns is highly recommended because it helps the
    campaigns achieve optimal performance.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        suggestion_info: a SmartCampaignSuggestionInfo instance with details
          about the business being advertised.

    Returns:
        a daily budget amount in micros.
    """
    sc_suggest_service: SmartCampaignSuggestServiceClient = client.get_service(
        "SmartCampaignSuggestService"
    )
    request: SuggestSmartCampaignBudgetOptionsRequest = client.get_type(
        "SuggestSmartCampaignBudgetOptionsRequest"
    )
    request.customer_id = customer_id
    # You can retrieve suggestions for an existing campaign by setting the
    # "campaign" field of the request equal to the resource name of a campaign
    # and leaving the rest of the request fields below unset:
    # request.campaign = INSERT_CAMPAIGN_RESOURCE_NAME_HERE

    # Since these suggestions are for a new campaign, we're going to
    # use the suggestion_info field instead.
    request.suggestion_info = suggestion_info

    # Issue a request to retrieve a budget suggestion.
    response: SuggestSmartCampaignBudgetOptionsResponse = (
        sc_suggest_service.suggest_smart_campaign_budget_options(
            request=request
        )
    )

    # Three tiers of options will be returned, a "low", "high" and
    # "recommended". Here we will use the "recommended" option. The amount is
    # specified in micros, where one million is equivalent to one currency unit.
    recommendation: SuggestSmartCampaignBudgetOptionsResponse.BudgetOption = (
        response.recommended
    )
    print(
        f"A daily budget amount of {recommendation.daily_amount_micros} micros "
        "was suggested, garnering an estimated minimum of "
        f"{recommendation.metrics.min_daily_clicks} clicks and an estimated "
        f"maximum of {recommendation.metrics.max_daily_clicks} per day."
    )

    return recommendation.daily_amount_micros
    # [END add_smart_campaign_1]


# [START add_smart_campaign_10]
def get_ad_suggestions(
    client: GoogleAdsClient,
    customer_id: str,
    suggestion_info: SmartCampaignSuggestionInfo,
) -> SmartCampaignAdInfo:
    """Retrieves creative suggestions for a Smart campaign ad.

    Using the SmartCampaignSuggestService to suggest creatives for new and
    existing Smart campaigns is highly recommended because it helps the
    campaigns achieve optimal performance.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        suggestion_info: a SmartCampaignSuggestionInfo instance with details
          about the business being advertised.

    Returns:
        a SmartCampaignAdInfo instance with suggested headlines and
            descriptions.
    """
    sc_suggest_service: SmartCampaignSuggestServiceClient = client.get_service(
        "SmartCampaignSuggestService"
    )
    request: SuggestSmartCampaignAdRequest = client.get_type(
        "SuggestSmartCampaignAdRequest"
    )
    request.customer_id = customer_id

    # Unlike the SuggestSmartCampaignBudgetOptions method, it's only possible
    # to use suggestion_info to retrieve ad creative suggestions.
    request.suggestion_info = suggestion_info

    # Issue a request to retrieve ad creative suggestions.
    response: SuggestSmartCampaignAdResponse = (
        sc_suggest_service.suggest_smart_campaign_ad(request=request)
    )

    # The SmartCampaignAdInfo object in the response contains a list of up to
    # three headlines and two descriptions. Note that some of the suggestions
    # may have empty strings as text. Before setting these on the ad you should
    # review them and filter out any empty values.
    ad_suggestions: SmartCampaignAdInfo = response.ad_info

    print("The following headlines were suggested:")
    headline: AdTextAsset
    for headline in ad_suggestions.headlines:
        print(f"\t{headline.text or '<None>'}")

    print("And the following descriptions were suggested:")
    description: AdTextAsset
    for description in ad_suggestions.descriptions:
        print(f"\t{description.text or '<None>'}")

    return ad_suggestions
    # [END add_smart_campaign_10]


# [START add_smart_campaign_2]
def create_campaign_budget_operation(
    client: GoogleAdsClient, customer_id: str, suggested_budget_amount: int
) -> MutateOperation:
    """Creates a MutateOperation that creates a new CampaignBudget.

    A temporary ID will be assigned to this campaign budget so that it can be
    referenced by other objects being created in the same Mutate request.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        suggested_budget_amount: a numeric daily budget amount in micros.

    Returns:
        a MutateOperation that creates a CampaignBudget.
    """
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    campaign_budget_operation: CampaignBudgetOperation = (
        mutate_operation.campaign_budget_operation
    )
    campaign_budget: CampaignBudget = campaign_budget_operation.create
    campaign_budget.name = f"Smart campaign budget #{uuid4()}"
    # A budget used for Smart campaigns must have the type SMART_CAMPAIGN.
    # Note that the field name "type_" is an implementation detail in Python,
    # the field's actual name is "type".
    campaign_budget.type_ = client.enums.BudgetTypeEnum.SMART_CAMPAIGN
    # The suggested budget amount from the SmartCampaignSuggestService is
    # a daily budget. We don't need to specify that here, because the budget
    # period already defaults to DAILY.
    campaign_budget.amount_micros = suggested_budget_amount
    # Set a temporary ID in the budget's resource name so it can be referenced
    # by the campaign in later steps.
    campaign_budget.resource_name = client.get_service(
        "CampaignBudgetService"
    ).campaign_budget_path(customer_id, _BUDGET_TEMPORARY_ID)

    return mutate_operation
    # [END add_smart_campaign_2]


# [START add_smart_campaign_3]
def create_smart_campaign_operation(
    client: GoogleAdsClient, customer_id: str
) -> MutateOperation:
    """Creates a MutateOperation that creates a new Smart campaign.

    A temporary ID will be assigned to this campaign so that it can
    be referenced by other objects being created in the same Mutate request.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.

    Returns:
        a MutateOperation that creates a campaign.
    """
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    campaign_operation: CampaignOperation = mutate_operation.campaign_operation
    campaign: Campaign = campaign_operation.create
    campaign.name = f"Smart campaign #{uuid4()}"
    # Set the campaign status as PAUSED. The campaign is the only entity in
    # the mutate request that should have its' status set.
    campaign.status = client.enums.CampaignStatusEnum.PAUSED
    # Campaign.AdvertisingChannelType is required to be SMART.
    campaign.advertising_channel_type = (
        client.enums.AdvertisingChannelTypeEnum.SMART
    )
    # Campaign.AdvertisingChannelSubType is required to be SMART_CAMPAIGN.
    campaign.advertising_channel_sub_type = (
        client.enums.AdvertisingChannelSubTypeEnum.SMART_CAMPAIGN
    )
    # Assign the resource name with a temporary ID.
    campaign.resource_name = client.get_service(
        "CampaignService"
    ).campaign_path(customer_id, _SMART_CAMPAIGN_TEMPORARY_ID)
    # Set the budget using the given budget resource name.
    campaign.campaign_budget = client.get_service(
        "CampaignBudgetService"
    ).campaign_budget_path(customer_id, _BUDGET_TEMPORARY_ID)

    # Declare whether or not this campaign serves political ads targeting the
    # EU. Valid values are:
    #   CONTAINS_EU_POLITICAL_ADVERTISING
    #   DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    campaign.contains_eu_political_advertising = (
        client.enums.EuPoliticalAdvertisingStatusEnum.DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    )

    return mutate_operation
    # [END add_smart_campaign_3]


# [START add_smart_campaign_4]
def create_smart_campaign_setting_operation(
    client: GoogleAdsClient,
    customer_id: str,
    business_profile_location: Optional[str],
    business_name: Optional[str],
) -> MutateOperation:
    """Creates a MutateOperation to create a new SmartCampaignSetting.

    SmartCampaignSettings are unique in that they only support UPDATE
    operations, which are used to update and create them. Below we will
    use a temporary ID in the resource name to associate it with the
    campaign created in the previous step.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        business_profile_location: the resource name of a Business Profile
          location.
        business_name: the name of a Business Profile.

    Returns:
        a MutateOperation that creates a SmartCampaignSetting.
    """
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    smart_campaign_setting_operation: SmartCampaignSettingOperation = (
        mutate_operation.smart_campaign_setting_operation
    )
    smart_campaign_setting: SmartCampaignSetting = (
        smart_campaign_setting_operation.update
    )
    # Set a temporary ID in the campaign setting's resource name to associate it
    # with the campaign created in the previous step.
    smart_campaign_setting.resource_name = client.get_service(
        "SmartCampaignSettingService"
    ).smart_campaign_setting_path(customer_id, _SMART_CAMPAIGN_TEMPORARY_ID)

    # Below we configure the SmartCampaignSetting using many of the same
    # details used to generate a budget suggestion.
    smart_campaign_setting.phone_number.country_code = _COUNTRY_CODE
    smart_campaign_setting.phone_number.phone_number = _PHONE_NUMBER
    smart_campaign_setting.final_url = _LANDING_PAGE_URL
    smart_campaign_setting.advertising_language_code = _LANGUAGE_CODE

    # Set either of the business_profile_location or business_name, depending on
    # whichever is provided.
    if business_profile_location:
        smart_campaign_setting.business_profile_location = (
            business_profile_location
        )
    else:
        smart_campaign_setting.business_name = business_name

    # Set the update mask on the operation. This is required since the smart
    # campaign setting is created in an UPDATE operation. Here the update
    # mask will be a list of all the fields that were set on the
    # SmartCampaignSetting.
    client.copy_from(
        smart_campaign_setting_operation.update_mask,
        protobuf_helpers.field_mask(None, smart_campaign_setting._pb),
    )

    return mutate_operation
    # [END add_smart_campaign_4]


# [START add_smart_campaign_8]
def create_campaign_criterion_operations(
    client: GoogleAdsClient,
    customer_id: str,
    keyword_theme_infos: List[KeywordThemeInfo],
    suggestion_info: SmartCampaignSuggestionInfo,
) -> List[MutateOperation]:
    """Creates a list of MutateOperations that create new campaign criteria.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        keyword_theme_infos: a list of KeywordThemeInfos.
        suggestion_info: A SmartCampaignSuggestionInfo instance.

    Returns:
        a list of MutateOperations that create new campaign criteria.
    """
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )

    operations: List[MutateOperation] = []
    info: KeywordThemeInfo
    for info in keyword_theme_infos:
        mutate_operation: MutateOperation = client.get_type("MutateOperation")
        campaign_criterion_operation: CampaignCriterionOperation = (
            mutate_operation.campaign_criterion_operation
        )
        campaign_criterion: CampaignCriterion = (
            campaign_criterion_operation.create
        )
        # Set the campaign ID to a temporary ID.
        campaign_criterion.campaign = campaign_service.campaign_path(
            customer_id, _SMART_CAMPAIGN_TEMPORARY_ID
        )
        # Set the keyword theme to the given KeywordThemeInfo.
        campaign_criterion.keyword_theme.CopyFrom(info)
        # Add the mutate operation to the list of other operations.
        operations.append(mutate_operation)

    # Create a location criterion for each location in the suggestion info
    # object to add corresponding location targeting to the Smart campaign
    location_info: LocationInfo
    for location_info in suggestion_info.location_list.locations:
        mutate_operation: MutateOperation = client.get_type("MutateOperation")
        campaign_criterion_operation: CampaignCriterionOperation = (
            mutate_operation.campaign_criterion_operation
        )
        campaign_criterion: CampaignCriterion = (
            campaign_criterion_operation.create
        )
        # Set the campaign ID to a temporary ID.
        campaign_criterion.campaign = campaign_service.campaign_path(
            customer_id, _SMART_CAMPAIGN_TEMPORARY_ID
        )
        # Set the location to the given location.
        campaign_criterion.location.CopyFrom(location_info)
        # Add the mutate operation to the list of other operations.
        operations.append(mutate_operation)

    return operations
    # [END add_smart_campaign_8]


# [START add_smart_campaign_5]
def create_ad_group_operation(
    client: GoogleAdsClient, customer_id: str
) -> MutateOperation:
    """Creates a MutateOperation that creates a new ad group.

    A temporary ID will be used in the campaign resource name for this
    ad group to associate it with the Smart campaign created in earlier steps.
    A temporary ID will also be used for its own resource name so that we can
    associate an ad group ad with it later in the process.

    Only one ad group can be created for a given Smart campaign.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.

    Returns:
        a MutateOperation that creates a new ad group.
    """
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    ad_group_operation: AdGroupOperation = mutate_operation.ad_group_operation
    ad_group: AdGroup = ad_group_operation.create
    # Set the ad group ID to a temporary ID.
    ad_group.resource_name = client.get_service("AdGroupService").ad_group_path(
        customer_id, _AD_GROUP_TEMPORARY_ID
    )
    ad_group.name = f"Smart campaign ad group #{uuid4()}"
    # Set the campaign ID to a temporary ID.
    ad_group.campaign = client.get_service("CampaignService").campaign_path(
        customer_id, _BUDGET_TEMPORARY_ID
    )
    # The ad group type must be set to SMART_CAMPAIGN_ADS.
    ad_group.type_ = client.enums.AdGroupTypeEnum.SMART_CAMPAIGN_ADS

    return mutate_operation
    # [END add_smart_campaign_5]


# [START add_smart_campaign_6]
def create_ad_group_ad_operation(
    client: GoogleAdsClient,
    customer_id: str,
    ad_suggestions: SmartCampaignAdInfo,
) -> MutateOperation:
    """Creates a MutateOperation that creates a new ad group ad.

    A temporary ID will be used in the ad group resource name for this
    ad group ad to associate it with the ad group created in earlier steps.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        ad_suggestions: a SmartCampaignAdInfo object with ad creative
          suggestions.

    Returns:
        a MutateOperation that creates a new ad group ad.
    """
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    ad_group_ad_operation: AdGroupAdOperation = (
        mutate_operation.ad_group_ad_operation
    )
    ad_group_ad: AdGroupAd = ad_group_ad_operation.create
    # Set the ad group ID to a temporary ID.
    ad_group_ad.ad_group = client.get_service("AdGroupService").ad_group_path(
        customer_id, _AD_GROUP_TEMPORARY_ID
    )
    # Set the type to SMART_CAMPAIGN_AD.
    ad_group_ad.ad.type_ = client.enums.AdTypeEnum.SMART_CAMPAIGN_AD
    ad: SmartCampaignAdInfo = ad_group_ad.ad.smart_campaign_ad

    # The SmartCampaignAdInfo object includes headlines and descriptions
    # retrieved from the SmartCampaignSuggestService.SuggestSmartCampaignAd
    # method. It's recommended that users review and approve or update these
    # creatives before they're set on the ad. It's possible that some or all of
    # these assets may contain empty texts, which should not be set on the ad
    # and instead should be replaced with meaningful texts from the user. Below
    # we just accept the creatives that were suggested while filtering out empty
    # assets. If no headlines or descriptions were suggested, then we manually
    # add some, otherwise this operation will generate an INVALID_ARGUMENT
    # error. Individual workflows will likely vary here.
    ad.headlines.extend(
        [asset for asset in ad_suggestions.headlines if asset.text]
    )
    num_missing_headlines: int = _REQUIRED_NUM_HEADLINES - len(ad.headlines)

    # If there are fewer headlines than are required, we manually add additional
    # headlines to make up for the difference.
    for i in range(num_missing_headlines):
        headline: AdTextAsset = client.get_type("AdTextAsset")
        headline.text = f"placeholder headline {i}"
        ad.headlines.append(headline)

    ad.descriptions.extend(
        asset for asset in ad_suggestions.descriptions if asset.text
    )
    num_missing_descriptions: int = _REQUIRED_NUM_DESCRIPTIONS - len(
        ad.descriptions
    )

    # If there are fewer descriptions than are required, we manually add
    # additional descriptions to make up for the difference.
    for i in range(num_missing_descriptions):
        description: AdTextAsset = client.get_type("AdTextAsset")
        description.text = f"placeholder description {i}"
        ad.descriptions.append(description)

    return mutate_operation
    # [END add_smart_campaign_6]


def print_response_details(response: MutateGoogleAdsResponse) -> None:
    """Prints the details of a MutateGoogleAdsResponse.

    Parses the "response" oneof field name and uses it to extract the new
    entity's name and resource name.

    Args:
        response: a MutateGoogleAdsResponse object.
    """
    # Parse the Mutate response to print details about the entities that
    # were created by the request.
    result: MutateOperationResponse
    for result in response.mutate_operation_responses:
        resource_type: str = "unrecognized"
        resource_name: str = "not found"

        if result._pb.HasField("campaign_budget_result"):
            resource_type = "CampaignBudget"
            resource_name = result.campaign_budget_result.resource_name
        elif result._pb.HasField("campaign_result"):
            resource_type = "Campaign"
            resource_name = result.campaign_result.resource_name
        elif result._pb.HasField("smart_campaign_setting_result"):
            resource_type = "SmartCampaignSettingResult"
            resource_name = result.smart_campaign_setting_result.resource_name
        elif result._pb.HasField("campaign_criterion_result"):
            resource_type = "CampaignCriterion"
            resource_name = result.campaign_criterion_result.resource_name
        elif result._pb.HasField("ad_group_result"):
            resource_type = "AdGroup"
            resource_name = result.ad_group_result.resource_name
        elif result._pb.HasField("ad_group_ad_result"):
            resource_type = "AdGroupAd"
            resource_name = result.ad_group_ad_result.resource_name

        print(
            f"Created a(n) {resource_type} with "
            f"resource_name: '{resource_name}'."
        )


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description=("Creates a Smart campaign.")
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-k",
        "--keyword_text",
        type=str,
        required=False,
        help=(
            "A keyword text used to retrieve keyword theme constant "
            "suggestions from the KeywordThemeConstantService. These keyword "
            "theme suggestions are generated using auto-completion data for "
            "the given text and may help improve the performance of "
            "the Smart campaign."
        ),
    )
    parser.add_argument(
        "-f",
        "--free_form_keyword_text",
        type=str,
        required=False,
        help=(
            "A keyword text used to create a freeform keyword theme, which is "
            "entirely user-specified and not derived from any suggestion "
            "service. Using free-form keyword themes is typically not "
            "recommended because they are less effective than suggested "
            "keyword themes, however they are useful in situations where a "
            "very specific term needs to be targeted."
        ),
    )
    group: argparse._MutuallyExclusiveGroup = (
        parser.add_mutually_exclusive_group(required=True)
    )
    group.add_argument(
        "-b",
        "--business_profile_location",
        type=str,
        help=(
            "Optional: Specify the resource name of a Business Profile location. This"
            "is required if a business name is not provided. It can be retrieved"
            "using the Business Profile API, see:"
            "https://developers.google.com/my-business/reference/businessinformation/rest/v1/accounts.locations"
            "or from the Business Profile UI (https://support.google.com/business/answer/10737668)."
        ),
    )
    group.add_argument(
        "-n",
        "--business_name",
        type=str,
        help=(
            "The name of a Business Profile business. This is required "
            "if a business profile location is not provided."
        ),
    )

    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.keyword_text,
            args.free_form_keyword_text,
            args.business_profile_location,
            args.business_name,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'Error with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import campaign_shared_set_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "CampaignSharedSet",
    },
)


class CampaignSharedSet(proto.Message):
    r"""CampaignSharedSets are used for managing the shared sets
    associated with a campaign.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the campaign shared set.
            Campaign shared set resource names have the form:

            ``customers/{customer_id}/campaignSharedSets/{campaign_id}~{shared_set_id}``
        campaign (str):
            Immutable. The campaign to which the campaign
            shared set belongs.

            This field is a member of `oneof`_ ``_campaign``.
        shared_set (str):
            Immutable. The shared set associated with the
            campaign. This may be a negative keyword shared
            set of another customer. This customer should be
            a manager of the other customer, otherwise the
            campaign shared set will exist but have no
            serving effect. Only negative keyword shared
            sets can be associated with Shopping campaigns.
            Only negative placement shared sets can be
            associated with Display mobile app campaigns.

            This field is a member of `oneof`_ ``_shared_set``.
        status (google.ads.googleads.v20.enums.types.CampaignSharedSetStatusEnum.CampaignSharedSetStatus):
            Output only. The status of this campaign
            shared set. Read only.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    shared_set: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    status: (
        campaign_shared_set_status.CampaignSharedSetStatusEnum.CampaignSharedSetStatus
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=campaign_shared_set_status.CampaignSharedSetStatusEnum.CampaignSharedSetStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "ExternalConversionSourceEnum",
    },
)


class ExternalConversionSourceEnum(proto.Message):
    r"""Container for enum describing the external conversion source
    that is associated with a ConversionAction.

    """

    class ExternalConversionSource(proto.Enum):
        r"""The external conversion source that is associated with a
        ConversionAction.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Represents value unknown in this version.
            WEBPAGE (2):
                Conversion that occurs when a user navigates
                to a particular webpage after viewing an ad;
                Displayed in Google Ads UI as 'Website'.
            ANALYTICS (3):
                Conversion that comes from linked Google
                Analytics goal or transaction; Displayed in
                Google Ads UI as 'Analytics'.
            UPLOAD (4):
                Website conversion that is uploaded through
                ConversionUploadService; Displayed in Google Ads
                UI as 'Import from clicks'.
            AD_CALL_METRICS (5):
                Conversion that occurs when a user clicks on
                a call extension directly on an ad; Displayed in
                Google Ads UI as 'Calls from ads'.
            WEBSITE_CALL_METRICS (6):
                Conversion that occurs when a user calls a
                dynamically-generated phone number (by installed
                javascript) from an advertiser's website after
                clicking on an ad; Displayed in Google Ads UI as
                'Calls from website'.
            STORE_VISITS (7):
                Conversion that occurs when a user visits an
                advertiser's retail store after clicking on a
                Google ad; Displayed in Google Ads UI as 'Store
                visits'.
            ANDROID_IN_APP (8):
                Conversion that occurs when a user takes an
                in-app action such as a purchase in an Android
                app; Displayed in Google Ads UI as 'Android
                in-app action'.
            IOS_IN_APP (9):
                Conversion that occurs when a user takes an
                in-app action such as a purchase in an iOS app;
                Displayed in Google Ads UI as 'iOS in-app
                action'.
            IOS_FIRST_OPEN (10):
                Conversion that occurs when a user opens an
                iOS app for the first time; Displayed in Google
                Ads UI as 'iOS app install (first open)'.
            APP_UNSPECIFIED (11):
                Legacy app conversions that do not have an
                AppPlatform provided; Displayed in Google Ads UI
                as 'Mobile app'.
            ANDROID_FIRST_OPEN (12):
                Conversion that occurs when a user opens an
                Android app for the first time; Displayed in
                Google Ads UI as 'Android app install (first
                open)'.
            UPLOAD_CALLS (13):
                Call conversion that is uploaded through
                ConversionUploadService; Displayed in Google Ads
                UI as 'Import from calls'.
            FIREBASE (14):
                Conversion that comes from a linked Firebase
                event; Displayed in Google Ads UI as 'Firebase'.
            CLICK_TO_CALL (15):
                Conversion that occurs when a user clicks on
                a mobile phone number; Displayed in Google Ads
                UI as 'Phone number clicks'.
            SALESFORCE (16):
                Conversion that comes from Salesforce;
                Displayed in Google Ads UI as 'Salesforce.com'.
            STORE_SALES_CRM (17):
                Conversion that comes from in-store purchases
                recorded by CRM; Displayed in Google Ads UI as
                'Store sales (data partner)'.
            STORE_SALES_PAYMENT_NETWORK (18):
                Conversion that comes from in-store purchases
                from payment network; Displayed in Google Ads UI
                as 'Store sales (payment network)'.
            GOOGLE_PLAY (19):
                Codeless Google Play conversion;
                Displayed in Google Ads UI as 'Google Play'.
            THIRD_PARTY_APP_ANALYTICS (20):
                Conversion that comes from a linked
                third-party app analytics event; Displayed in
                Google Ads UI as 'Third-party app analytics'.
            GOOGLE_ATTRIBUTION (21):
                Conversion that is controlled by Google
                Attribution.
            STORE_SALES_DIRECT_UPLOAD (23):
                Store Sales conversion based on first-party
                or third-party merchant data uploads. Displayed
                in Google Ads UI as 'Store sales (direct
                upload)'.
            STORE_SALES (24):
                Store Sales conversion based on first-party
                or third-party merchant data uploads and/or from
                in-store purchases using cards from payment
                networks. Displayed in Google Ads UI as 'Store
                sales'.
            SEARCH_ADS_360 (25):
                Conversions imported from Search Ads 360
                Floodlight data.
            GOOGLE_HOSTED (27):
                Conversions that track local actions from
                Google's products and services after interacting
                with an ad.
            FLOODLIGHT (29):
                Conversions reported by Floodlight tags.
            ANALYTICS_SEARCH_ADS_360 (31):
                Conversions that come from Google Analytics
                specifically for Search Ads
                360. Displayed in Google Ads UI as Analytics
                    (SA360).
            FIREBASE_SEARCH_ADS_360 (33):
                Conversion that comes from a linked Firebase
                event for Search Ads 360.
            DISPLAY_AND_VIDEO_360_FLOODLIGHT (34):
                Conversion that is reported by Floodlight for
                DV360.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        WEBPAGE = 2
        ANALYTICS = 3
        UPLOAD = 4
        AD_CALL_METRICS = 5
        WEBSITE_CALL_METRICS = 6
        STORE_VISITS = 7
        ANDROID_IN_APP = 8
        IOS_IN_APP = 9
        IOS_FIRST_OPEN = 10
        APP_UNSPECIFIED = 11
        ANDROID_FIRST_OPEN = 12
        UPLOAD_CALLS = 13
        FIREBASE = 14
        CLICK_TO_CALL = 15
        SALESFORCE = 16
        STORE_SALES_CRM = 17
        STORE_SALES_PAYMENT_NETWORK = 18
        GOOGLE_PLAY = 19
        THIRD_PARTY_APP_ANALYTICS = 20
        GOOGLE_ATTRIBUTION = 21
        STORE_SALES_DIRECT_UPLOAD = 23
        STORE_SALES = 24
        SEARCH_ADS_360 = 25
        GOOGLE_HOSTED = 27
        FLOODLIGHT = 29
        ANALYTICS_SEARCH_ADS_360 = 31
        FIREBASE_SEARCH_ADS_360 = 33
        DISPLAY_AND_VIDEO_360_FLOODLIGHT = 34


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.common.types import dates
from google.ads.googleads.v21.enums.types import invoice_type
from google.ads.googleads.v21.enums.types import month_of_year


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "Invoice",
    },
)


class Invoice(proto.Message):
    r"""An invoice. All invoice information is snapshotted to match
    the PDF invoice. For invoices older than the launch of
    InvoiceService, the snapshotted information may not match the
    PDF invoice.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the invoice. Multiple
            customers can share a given invoice, so multiple resource
            names may point to the same invoice. Invoice resource names
            have the form:

            ``customers/{customer_id}/invoices/{invoice_id}``
        id (str):
            Output only. The ID of the invoice. It
            appears on the invoice PDF as "Invoice number".

            This field is a member of `oneof`_ ``_id``.
        type_ (google.ads.googleads.v21.enums.types.InvoiceTypeEnum.InvoiceType):
            Output only. The type of invoice.
        billing_setup (str):
            Output only. The resource name of this invoice's billing
            setup.

            ``customers/{customer_id}/billingSetups/{billing_setup_id}``

            This field is a member of `oneof`_ ``_billing_setup``.
        payments_account_id (str):
            Output only. A 16 digit ID used to identify
            the payments account associated with the billing
            setup, for example, "1234-5678-9012-3456". It
            appears on the invoice PDF as "Billing Account
            Number".

            This field is a member of `oneof`_ ``_payments_account_id``.
        payments_profile_id (str):
            Output only. A 12 digit ID used to identify
            the payments profile associated with the billing
            setup, for example, "1234-5678-9012". It appears
            on the invoice PDF as "Billing ID".

            This field is a member of `oneof`_ ``_payments_profile_id``.
        issue_date (str):
            Output only. The issue date in yyyy-mm-dd
            format. It appears on the invoice PDF as either
            "Issue date" or "Invoice date".

            This field is a member of `oneof`_ ``_issue_date``.
        due_date (str):
            Output only. The due date in yyyy-mm-dd
            format.

            This field is a member of `oneof`_ ``_due_date``.
        service_date_range (google.ads.googleads.v21.common.types.DateRange):
            Output only. The service period date range of
            this invoice. The end date is inclusive.
        currency_code (str):
            Output only. The currency code. All costs are
            returned in this currency. A subset of the
            currency codes derived from the ISO 4217
            standard is supported.

            This field is a member of `oneof`_ ``_currency_code``.
        adjustments_subtotal_amount_micros (int):
            Output only. The pretax subtotal amount of
            invoice level adjustments, in micros.
        adjustments_tax_amount_micros (int):
            Output only. The sum of taxes on the invoice
            level adjustments, in micros.
        adjustments_total_amount_micros (int):
            Output only. The total amount of invoice
            level adjustments, in micros.
        regulatory_costs_subtotal_amount_micros (int):
            Output only. The pretax subtotal amount of
            invoice level regulatory costs, in micros.
        regulatory_costs_tax_amount_micros (int):
            Output only. The sum of taxes on the invoice
            level regulatory costs, in micros.
        regulatory_costs_total_amount_micros (int):
            Output only. The total amount of invoice
            level regulatory costs, in micros.
        export_charge_subtotal_amount_micros (int):
            Output only. The pretax subtotal amount of
            invoice level export charges, in micros.

            This field is a member of `oneof`_ ``_export_charge_subtotal_amount_micros``.
        export_charge_tax_amount_micros (int):
            Output only. The sum of taxes on the invoice
            level export charges, in micros.

            This field is a member of `oneof`_ ``_export_charge_tax_amount_micros``.
        export_charge_total_amount_micros (int):
            Output only. The total amount of invoice
            level export charges, in micros.

            This field is a member of `oneof`_ ``_export_charge_total_amount_micros``.
        subtotal_amount_micros (int):
            Output only. The pretax subtotal amount, in micros. This is
            equal to the sum of the AccountBudgetSummary subtotal
            amounts and Invoice.adjustments_subtotal_amount_micros.

            This field is a member of `oneof`_ ``_subtotal_amount_micros``.
        tax_amount_micros (int):
            Output only. The sum of all taxes on the
            invoice, in micros. This equals the sum of the
            AccountBudgetSummary tax amounts, plus taxes not
            associated with a specific account budget.

            This field is a member of `oneof`_ ``_tax_amount_micros``.
        total_amount_micros (int):
            Output only. The total amount, in micros. This equals the
            sum of Invoice.subtotal_amount_micros,
            Invoice.tax_amount_micros,
            Invoice.regulatory_costs_subtotal_amount_micros, and
            Invoice.export_charge_subtotal_amount_micros (which is
            separated into a separate line item starting with V14.1).

            This field is a member of `oneof`_ ``_total_amount_micros``.
        corrected_invoice (str):
            Output only. The resource name of the original invoice
            corrected, wrote off, or canceled by this invoice, if
            applicable. If ``corrected_invoice`` is set,
            ``replaced_invoices`` will not be set. Invoice resource
            names have the form:

            ``customers/{customer_id}/invoices/{invoice_id}``

            This field is a member of `oneof`_ ``_corrected_invoice``.
        replaced_invoices (MutableSequence[str]):
            Output only. The resource name of the original invoice(s)
            being rebilled or replaced by this invoice, if applicable.
            There might be multiple replaced invoices due to invoice
            consolidation. The replaced invoices may not belong to the
            same payments account. If ``replaced_invoices`` is set,
            ``corrected_invoice`` will not be set. Invoice resource
            names have the form:

            ``customers/{customer_id}/invoices/{invoice_id}``
        pdf_url (str):
            Output only. The URL to a PDF copy of the
            invoice. Users need to pass in their OAuth token
            to request the PDF with this URL.

            This field is a member of `oneof`_ ``_pdf_url``.
        account_budget_summaries (MutableSequence[google.ads.googleads.v21.resources.types.Invoice.AccountBudgetSummary]):
            Output only. The list of summarized account
            budget information associated with this invoice.
        account_summaries (MutableSequence[google.ads.googleads.v21.resources.types.Invoice.AccountSummary]):
            Output only. The list of summarized account
            information associated with this invoice.
    """

    class AccountSummary(proto.Message):
        r"""Represents a summarized view at account level.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            customer (str):
                Output only. The account associated with the
                account summary.

                This field is a member of `oneof`_ ``_customer``.
            billing_correction_subtotal_amount_micros (int):
                Output only. Pretax billing correction
                subtotal amount, in micros.

                This field is a member of `oneof`_ ``_billing_correction_subtotal_amount_micros``.
            billing_correction_tax_amount_micros (int):
                Output only. Tax on billing correction, in
                micros.

                This field is a member of `oneof`_ ``_billing_correction_tax_amount_micros``.
            billing_correction_total_amount_micros (int):
                Output only. Total billing correction amount,
                in micros.

                This field is a member of `oneof`_ ``_billing_correction_total_amount_micros``.
            coupon_adjustment_subtotal_amount_micros (int):
                Output only. Pretax coupon adjustment
                subtotal amount, in micros.

                This field is a member of `oneof`_ ``_coupon_adjustment_subtotal_amount_micros``.
            coupon_adjustment_tax_amount_micros (int):
                Output only. Tax on coupon adjustment, in
                micros.

                This field is a member of `oneof`_ ``_coupon_adjustment_tax_amount_micros``.
            coupon_adjustment_total_amount_micros (int):
                Output only. Total coupon adjustment amount,
                in micros.

                This field is a member of `oneof`_ ``_coupon_adjustment_total_amount_micros``.
            excess_credit_adjustment_subtotal_amount_micros (int):
                Output only. Pretax excess credit adjustment
                subtotal amount, in micros.

                This field is a member of `oneof`_ ``_excess_credit_adjustment_subtotal_amount_micros``.
            excess_credit_adjustment_tax_amount_micros (int):
                Output only. Tax on excess credit adjustment,
                in micros.

                This field is a member of `oneof`_ ``_excess_credit_adjustment_tax_amount_micros``.
            excess_credit_adjustment_total_amount_micros (int):
                Output only. Total excess credit adjustment
                amount, in micros.

                This field is a member of `oneof`_ ``_excess_credit_adjustment_total_amount_micros``.
            regulatory_costs_subtotal_amount_micros (int):
                Output only. Pretax regulatory costs subtotal
                amount, in micros.

                This field is a member of `oneof`_ ``_regulatory_costs_subtotal_amount_micros``.
            regulatory_costs_tax_amount_micros (int):
                Output only. Tax on regulatory costs, in
                micros.

                This field is a member of `oneof`_ ``_regulatory_costs_tax_amount_micros``.
            regulatory_costs_total_amount_micros (int):
                Output only. Total regulatory costs amount,
                in micros.

                This field is a member of `oneof`_ ``_regulatory_costs_total_amount_micros``.
            export_charge_subtotal_amount_micros (int):
                Output only. Pretax export charge subtotal
                amount, in micros.

                This field is a member of `oneof`_ ``_export_charge_subtotal_amount_micros``.
            export_charge_tax_amount_micros (int):
                Output only. Tax on export charge, in micros.

                This field is a member of `oneof`_ ``_export_charge_tax_amount_micros``.
            export_charge_total_amount_micros (int):
                Output only. Total export charge amount, in
                micros.

                This field is a member of `oneof`_ ``_export_charge_total_amount_micros``.
            subtotal_amount_micros (int):
                Output only. Total pretax subtotal amount
                attributable to the account during the service
                period, in micros.

                This field is a member of `oneof`_ ``_subtotal_amount_micros``.
            tax_amount_micros (int):
                Output only. Total tax amount attributable to
                the account during the service period, in
                micros.

                This field is a member of `oneof`_ ``_tax_amount_micros``.
            total_amount_micros (int):
                Output only. Total amount attributable to the account during
                the service period, in micros. This equals the sum of the
                subtotal_amount_micros and tax_amount_micros.

                This field is a member of `oneof`_ ``_total_amount_micros``.
        """

        customer: str = proto.Field(
            proto.STRING,
            number=1,
            optional=True,
        )
        billing_correction_subtotal_amount_micros: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )
        billing_correction_tax_amount_micros: int = proto.Field(
            proto.INT64,
            number=3,
            optional=True,
        )
        billing_correction_total_amount_micros: int = proto.Field(
            proto.INT64,
            number=4,
            optional=True,
        )
        coupon_adjustment_subtotal_amount_micros: int = proto.Field(
            proto.INT64,
            number=5,
            optional=True,
        )
        coupon_adjustment_tax_amount_micros: int = proto.Field(
            proto.INT64,
            number=6,
            optional=True,
        )
        coupon_adjustment_total_amount_micros: int = proto.Field(
            proto.INT64,
            number=7,
            optional=True,
        )
        excess_credit_adjustment_subtotal_amount_micros: int = proto.Field(
            proto.INT64,
            number=8,
            optional=True,
        )
        excess_credit_adjustment_tax_amount_micros: int = proto.Field(
            proto.INT64,
            number=9,
            optional=True,
        )
        excess_credit_adjustment_total_amount_micros: int = proto.Field(
            proto.INT64,
            number=10,
            optional=True,
        )
        regulatory_costs_subtotal_amount_micros: int = proto.Field(
            proto.INT64,
            number=11,
            optional=True,
        )
        regulatory_costs_tax_amount_micros: int = proto.Field(
            proto.INT64,
            number=12,
            optional=True,
        )
        regulatory_costs_total_amount_micros: int = proto.Field(
            proto.INT64,
            number=13,
            optional=True,
        )
        export_charge_subtotal_amount_micros: int = proto.Field(
            proto.INT64,
            number=17,
            optional=True,
        )
        export_charge_tax_amount_micros: int = proto.Field(
            proto.INT64,
            number=18,
            optional=True,
        )
        export_charge_total_amount_micros: int = proto.Field(
            proto.INT64,
            number=19,
            optional=True,
        )
        subtotal_amount_micros: int = proto.Field(
            proto.INT64,
            number=14,
            optional=True,
        )
        tax_amount_micros: int = proto.Field(
            proto.INT64,
            number=15,
            optional=True,
        )
        total_amount_micros: int = proto.Field(
            proto.INT64,
            number=16,
            optional=True,
        )

    class AccountBudgetSummary(proto.Message):
        r"""Represents a summarized account budget billable cost.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            customer (str):
                Output only. The resource name of the customer associated
                with this account budget. This contains the customer ID,
                which appears on the invoice PDF as "Account ID". Customer
                resource names have the form:

                ``customers/{customer_id}``

                This field is a member of `oneof`_ ``_customer``.
            customer_descriptive_name (str):
                Output only. The descriptive name of the
                account budget's customer. It appears on the
                invoice PDF as "Account".

                This field is a member of `oneof`_ ``_customer_descriptive_name``.
            account_budget (str):
                Output only. The resource name of the account budget
                associated with this summarized billable cost. AccountBudget
                resource names have the form:

                ``customers/{customer_id}/accountBudgets/{account_budget_id}``

                This field is a member of `oneof`_ ``_account_budget``.
            account_budget_name (str):
                Output only. The name of the account budget.
                It appears on the invoice PDF as "Account
                budget".

                This field is a member of `oneof`_ ``_account_budget_name``.
            purchase_order_number (str):
                Output only. The purchase order number of the
                account budget. It appears on the invoice PDF as
                "Purchase order".

                This field is a member of `oneof`_ ``_purchase_order_number``.
            subtotal_amount_micros (int):
                Output only. The pretax subtotal amount
                attributable to this budget during the service
                period, in micros.

                This field is a member of `oneof`_ ``_subtotal_amount_micros``.
            tax_amount_micros (int):
                Output only. The tax amount attributable to
                this budget during the service period, in
                micros.

                This field is a member of `oneof`_ ``_tax_amount_micros``.
            total_amount_micros (int):
                Output only. The total amount attributable to
                this budget during the service period, in
                micros. This equals the sum of the account
                budget subtotal amount and the account budget
                tax amount.

                This field is a member of `oneof`_ ``_total_amount_micros``.
            billable_activity_date_range (google.ads.googleads.v21.common.types.DateRange):
                Output only. The billable activity date range
                of the account budget, within the service date
                range of this invoice. The end date is
                inclusive. This can be different from the
                account budget's start and end time.
            served_amount_micros (int):
                Output only. The pretax served amount
                attributable to this budget during the service
                period, in micros. This is only useful to
                reconcile invoice and delivery data.

                This field is a member of `oneof`_ ``_served_amount_micros``.
            billed_amount_micros (int):
                Output only. The pretax billed amount
                attributable to this budget during the service
                period, in micros. This does not account for any
                adjustments.

                This field is a member of `oneof`_ ``_billed_amount_micros``.
            overdelivery_amount_micros (int):
                Output only. The pretax overdelivery amount
                attributable to this budget during the service
                period, in micros (negative value).

                This field is a member of `oneof`_ ``_overdelivery_amount_micros``.
            invalid_activity_amount_micros (int):
                Output only. The pretax invalid activity
                amount attributable to this budget in previous
                months, in micros (negative value).

                This field is a member of `oneof`_ ``_invalid_activity_amount_micros``.
            invalid_activity_summaries (MutableSequence[google.ads.googleads.v21.resources.types.Invoice.InvalidActivitySummary]):
                Output only. The list of summarized invalid
                activity credits with original linkages.
        """

        customer: str = proto.Field(
            proto.STRING,
            number=10,
            optional=True,
        )
        customer_descriptive_name: str = proto.Field(
            proto.STRING,
            number=11,
            optional=True,
        )
        account_budget: str = proto.Field(
            proto.STRING,
            number=12,
            optional=True,
        )
        account_budget_name: str = proto.Field(
            proto.STRING,
            number=13,
            optional=True,
        )
        purchase_order_number: str = proto.Field(
            proto.STRING,
            number=14,
            optional=True,
        )
        subtotal_amount_micros: int = proto.Field(
            proto.INT64,
            number=15,
            optional=True,
        )
        tax_amount_micros: int = proto.Field(
            proto.INT64,
            number=16,
            optional=True,
        )
        total_amount_micros: int = proto.Field(
            proto.INT64,
            number=17,
            optional=True,
        )
        billable_activity_date_range: dates.DateRange = proto.Field(
            proto.MESSAGE,
            number=9,
            message=dates.DateRange,
        )
        served_amount_micros: int = proto.Field(
            proto.INT64,
            number=18,
            optional=True,
        )
        billed_amount_micros: int = proto.Field(
            proto.INT64,
            number=19,
            optional=True,
        )
        overdelivery_amount_micros: int = proto.Field(
            proto.INT64,
            number=20,
            optional=True,
        )
        invalid_activity_amount_micros: int = proto.Field(
            proto.INT64,
            number=21,
            optional=True,
        )
        invalid_activity_summaries: MutableSequence[
            "Invoice.InvalidActivitySummary"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=22,
            message="Invoice.InvalidActivitySummary",
        )

    class InvalidActivitySummary(proto.Message):
        r"""Details about the invalid activity for the invoice that
        contain additional details about invoice against which
        corrections are made.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            original_month_of_service (google.ads.googleads.v21.enums.types.MonthOfYearEnum.MonthOfYear):
                Output only. Original month of service
                related to this invalid activity credit.

                This field is a member of `oneof`_ ``_original_month_of_service``.
            original_year_of_service (str):
                Output only. Original year of service related
                to this invalid activity credit.

                This field is a member of `oneof`_ ``_original_year_of_service``.
            original_invoice_id (str):
                Output only. Original invoice number related
                to this invalid activity credit.

                This field is a member of `oneof`_ ``_original_invoice_id``.
            original_account_budget_name (str):
                Output only. Original account budget name
                related to this invalid activity credit.

                This field is a member of `oneof`_ ``_original_account_budget_name``.
            original_purchase_order_number (str):
                Output only. Original purchase order number
                related to this invalid activity credit.

                This field is a member of `oneof`_ ``_original_purchase_order_number``.
            amount_micros (int):
                Output only. Invalid activity amount in
                micros.

                This field is a member of `oneof`_ ``_amount_micros``.
        """

        original_month_of_service: month_of_year.MonthOfYearEnum.MonthOfYear = (
            proto.Field(
                proto.ENUM,
                number=1,
                optional=True,
                enum=month_of_year.MonthOfYearEnum.MonthOfYear,
            )
        )
        original_year_of_service: str = proto.Field(
            proto.STRING,
            number=2,
            optional=True,
        )
        original_invoice_id: str = proto.Field(
            proto.STRING,
            number=3,
            optional=True,
        )
        original_account_budget_name: str = proto.Field(
            proto.STRING,
            number=4,
            optional=True,
        )
        original_purchase_order_number: str = proto.Field(
            proto.STRING,
            number=5,
            optional=True,
        )
        amount_micros: int = proto.Field(
            proto.INT64,
            number=6,
            optional=True,
        )

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: str = proto.Field(
        proto.STRING,
        number=25,
        optional=True,
    )
    type_: invoice_type.InvoiceTypeEnum.InvoiceType = proto.Field(
        proto.ENUM,
        number=3,
        enum=invoice_type.InvoiceTypeEnum.InvoiceType,
    )
    billing_setup: str = proto.Field(
        proto.STRING,
        number=26,
        optional=True,
    )
    payments_account_id: str = proto.Field(
        proto.STRING,
        number=27,
        optional=True,
    )
    payments_profile_id: str = proto.Field(
        proto.STRING,
        number=28,
        optional=True,
    )
    issue_date: str = proto.Field(
        proto.STRING,
        number=29,
        optional=True,
    )
    due_date: str = proto.Field(
        proto.STRING,
        number=30,
        optional=True,
    )
    service_date_range: dates.DateRange = proto.Field(
        proto.MESSAGE,
        number=9,
        message=dates.DateRange,
    )
    currency_code: str = proto.Field(
        proto.STRING,
        number=31,
        optional=True,
    )
    adjustments_subtotal_amount_micros: int = proto.Field(
        proto.INT64,
        number=19,
    )
    adjustments_tax_amount_micros: int = proto.Field(
        proto.INT64,
        number=20,
    )
    adjustments_total_amount_micros: int = proto.Field(
        proto.INT64,
        number=21,
    )
    regulatory_costs_subtotal_amount_micros: int = proto.Field(
        proto.INT64,
        number=22,
    )
    regulatory_costs_tax_amount_micros: int = proto.Field(
        proto.INT64,
        number=23,
    )
    regulatory_costs_total_amount_micros: int = proto.Field(
        proto.INT64,
        number=24,
    )
    export_charge_subtotal_amount_micros: int = proto.Field(
        proto.INT64,
        number=40,
        optional=True,
    )
    export_charge_tax_amount_micros: int = proto.Field(
        proto.INT64,
        number=41,
        optional=True,
    )
    export_charge_total_amount_micros: int = proto.Field(
        proto.INT64,
        number=42,
        optional=True,
    )
    subtotal_amount_micros: int = proto.Field(
        proto.INT64,
        number=33,
        optional=True,
    )
    tax_amount_micros: int = proto.Field(
        proto.INT64,
        number=34,
        optional=True,
    )
    total_amount_micros: int = proto.Field(
        proto.INT64,
        number=35,
        optional=True,
    )
    corrected_invoice: str = proto.Field(
        proto.STRING,
        number=36,
        optional=True,
    )
    replaced_invoices: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=37,
    )
    pdf_url: str = proto.Field(
        proto.STRING,
        number=38,
        optional=True,
    )
    account_budget_summaries: MutableSequence[AccountBudgetSummary] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=18,
            message=AccountBudgetSummary,
        )
    )
    account_summaries: MutableSequence[AccountSummary] = proto.RepeatedField(
        proto.MESSAGE,
        number=39,
        message=AccountSummary,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "AdTypeEnum",
    },
)


class AdTypeEnum(proto.Message):
    r"""Container for enum describing possible types of an ad."""

    class AdType(proto.Enum):
        r"""The possible types of an ad.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                The received value is not known in this
                version.
                This is a response-only value.
            TEXT_AD (2):
                The ad is a text ad.
            EXPANDED_TEXT_AD (3):
                The ad is an expanded text ad.
            EXPANDED_DYNAMIC_SEARCH_AD (7):
                The ad is an expanded dynamic search ad.
            HOTEL_AD (8):
                The ad is a hotel ad.
            SHOPPING_SMART_AD (9):
                The ad is a Smart Shopping ad.
            SHOPPING_PRODUCT_AD (10):
                The ad is a standard Shopping ad.
            VIDEO_AD (12):
                The ad is a video ad.
            IMAGE_AD (14):
                This ad is an Image ad.
            RESPONSIVE_SEARCH_AD (15):
                The ad is a responsive search ad.
            LEGACY_RESPONSIVE_DISPLAY_AD (16):
                The ad is a legacy responsive display ad.
            APP_AD (17):
                The ad is an app ad.
            LEGACY_APP_INSTALL_AD (18):
                The ad is a legacy app install ad.
            RESPONSIVE_DISPLAY_AD (19):
                The ad is a responsive display ad.
            LOCAL_AD (20):
                The ad is a local ad.
            HTML5_UPLOAD_AD (21):
                The ad is a display upload ad with the HTML5_UPLOAD_AD
                product type.
            DYNAMIC_HTML5_AD (22):
                The ad is a display upload ad with one of the
                DYNAMIC_HTML5_\* product types.
            APP_ENGAGEMENT_AD (23):
                The ad is an app engagement ad.
            SHOPPING_COMPARISON_LISTING_AD (24):
                The ad is a Shopping Comparison Listing ad.
            VIDEO_BUMPER_AD (25):
                Video bumper ad.
            VIDEO_NON_SKIPPABLE_IN_STREAM_AD (26):
                Video non-skippable in-stream ad.
            VIDEO_TRUEVIEW_IN_STREAM_AD (29):
                Video TrueView in-stream ad.
            VIDEO_RESPONSIVE_AD (30):
                Video responsive ad.
            SMART_CAMPAIGN_AD (31):
                Smart campaign ad.
            CALL_AD (32):
                Call ad.
            APP_PRE_REGISTRATION_AD (33):
                Universal app pre-registration ad.
            IN_FEED_VIDEO_AD (34):
                In-feed video ad.
            DEMAND_GEN_MULTI_ASSET_AD (40):
                Demand Gen multi asset ad.
            DEMAND_GEN_CAROUSEL_AD (41):
                Demand Gen carousel ad.
            TRAVEL_AD (37):
                Travel ad.
            DEMAND_GEN_VIDEO_RESPONSIVE_AD (42):
                Demand Gen video responsive ad.
            DEMAND_GEN_PRODUCT_AD (39):
                Demand Gen product ad.
            YOUTUBE_AUDIO_AD (44):
                YouTube Audio ad.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        TEXT_AD = 2
        EXPANDED_TEXT_AD = 3
        EXPANDED_DYNAMIC_SEARCH_AD = 7
        HOTEL_AD = 8
        SHOPPING_SMART_AD = 9
        SHOPPING_PRODUCT_AD = 10
        VIDEO_AD = 12
        IMAGE_AD = 14
        RESPONSIVE_SEARCH_AD = 15
        LEGACY_RESPONSIVE_DISPLAY_AD = 16
        APP_AD = 17
        LEGACY_APP_INSTALL_AD = 18
        RESPONSIVE_DISPLAY_AD = 19
        LOCAL_AD = 20
        HTML5_UPLOAD_AD = 21
        DYNAMIC_HTML5_AD = 22
        APP_ENGAGEMENT_AD = 23
        SHOPPING_COMPARISON_LISTING_AD = 24
        VIDEO_BUMPER_AD = 25
        VIDEO_NON_SKIPPABLE_IN_STREAM_AD = 26
        VIDEO_TRUEVIEW_IN_STREAM_AD = 29
        VIDEO_RESPONSIVE_AD = 30
        SMART_CAMPAIGN_AD = 31
        CALL_AD = 32
        APP_PRE_REGISTRATION_AD = 33
        IN_FEED_VIDEO_AD = 34
        DEMAND_GEN_MULTI_ASSET_AD = 40
        DEMAND_GEN_CAROUSEL_AD = 41
        TRAVEL_AD = 37
        DEMAND_GEN_VIDEO_RESPONSIVE_AD = 42
        DEMAND_GEN_PRODUCT_AD = 39
        YOUTUBE_AUDIO_AD = 44


__all__ = tuple(sorted(__protobuf__.manifest))
